@@ -5,7 +5,7 @@
 # Author: Paul McCarthy <pauldmccarthy@gmail.com>
 
 """This module provides the :class:`OverlyDisplyaToolBar`, a
-:class:`.FSLeyesToolBar` containing controls for changing the display settings
+:class:`.ControlToolBar` containing controls for changing the display settings
 of the currently selected overlay.
 """
 
@@ -15,10 +15,12 @@ import os.path as op
 
 import wx
 
+import fsl.data.image                 as fslimage
 import fsleyes_props                  as props
+import fsleyes_widgets                as fwidgets
 import fsleyes_widgets.utils.typedict as td
-
-import fsleyes.toolbar                as fsltoolbar
+import fsleyes.controls.controlpanel  as ctrlpanel
+import fsleyes.views.canvaspanel      as canvaspanel
 import fsleyes.icons                  as icons
 import fsleyes.tooltips               as fsltooltips
 import fsleyes.actions                as actions
@@ -29,8 +31,8 @@ import fsleyes.strings                as strings
 log = logging.getLogger(__name__)
 
 
-class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
-    """The ``OverlyDisplyaToolBar`` is a :class:`.FSLeyesToolBar` containing
+class OverlayDisplayToolBar(ctrlpanel.ControlToolBar):
+    """The ``OverlyDisplyaToolBar`` is a :class:`.ControlToolBar` containing
     controls which allow the user to change the display settings of the
     currently selected overlay (as defined by the
     :attr:`.DisplayContext.selectedOverlay` property). The display settings
@@ -46,7 +48,7 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
 
 
     The specific controls which are displayed are defined in the
-    :attr:`_TOOLBAR_PROPS` dictionary, and are created by the following
+    :attr:`self.__widgetSpecs` dictionary, and are created by the following
     methods:
 
     .. autosummary::
@@ -64,9 +66,22 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
        _OverlayDisplayToolBar__makeFreesurferOptsTools
        _OverlayDisplayToolBar__makeTensorOptsTools
        _OverlayDisplayToolBar__makeSHOptsTools
+       _OverlayDisplayToolBar__makeMIPOptsTools
+       _OverlayDisplayToolBar__makeTractogramOptsTools
     """
 
-    def __init__(self, parent, overlayList, displayCtx, frame, viewPanel):
+
+    @staticmethod
+    def supportedViews():
+        """Overrides :meth:`.ControlMixin.supportedViews`. The
+        ``OverlayDisplayToolBar`` is only intended to be added to
+        :class:`.OrthoPanel`, :class:`.LightBoxPanel`, or
+        :class:`.Scene3DPanel` views.
+        """
+        return [canvaspanel.CanvasPanel]
+
+
+    def __init__(self, parent, overlayList, displayCtx, viewPanel):
         """Create an ``OverlyDisplyaToolBar``.
 
         :arg parent:      The :mod:`wx` parent object.
@@ -75,19 +90,17 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
 
         :arg displayCtx:  The :class:`.DisplayContext` instance.
 
-        :arg frame:       The :class:`.FSLeyesFrame` instance.
-
         :arg viewPanel:   The :class:`.ViewPanel` which this
                           ``OverlayDisplayToolBar`` is owned by.
         """
 
-        fsltoolbar.FSLeyesToolBar.__init__(self,
-                                           parent,
-                                           overlayList,
-                                           displayCtx,
-                                           frame,
-                                           height=24,
-                                           kbFocus=True)
+        ctrlpanel.ControlToolBar.__init__(self,
+                                          parent,
+                                          overlayList,
+                                          displayCtx,
+                                          viewPanel,
+                                          height=24,
+                                          kbFocus=True)
 
         self.__viewPanel      = viewPanel
         self.__currentOverlay = None
@@ -101,13 +114,14 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
             self.name,
             self.__selectedOverlayChanged)
 
+        self.__generateWidgetSpecs()
         self.__selectedOverlayChanged()
 
 
     def destroy(self):
         """Must be called when this ``OverlyDisplyaToolBar`` is no longer
         needed. Removes some property listeners, and calls the
-        :meth:`.FSLeyesToolBar.destroy` method.
+        :meth:`.ControlToolBar.destroy` method.
         """
 
         self.overlayList.removeListener('overlays',        self.name)
@@ -123,7 +137,7 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
         self.__currentOverlay = None
         self.__viewPanel      = None
 
-        fsltoolbar.FSLeyesToolBar.destroy(self)
+        ctrlpanel.ControlToolBar.destroy(self)
 
 
     def __showTools(self, overlay):
@@ -135,8 +149,11 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
 
         # See long comment at bottom
         def destroyOldTools():
+            if self.destroyed:
+                return
             for t in oldTools:
-                t.Destroy()
+                if fwidgets.isalive(t):
+                    t.Destroy()
 
         for t in oldTools:
             t.Show(False)
@@ -232,27 +249,33 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
         viewPanel = self.__viewPanel
 
         # Display settings
-        nameSpec  = _TOOLBAR_PROPS[display, 'name']
-        typeSpec  = _TOOLBAR_PROPS[display, 'overlayType']
-        alphaSpec = _TOOLBAR_PROPS[display, 'alpha']
-        briSpec   = _TOOLBAR_PROPS[display, 'brightness']
-        conSpec   = _TOOLBAR_PROPS[display, 'contrast']
+        nameSpec  = self.__widgetSpecs[display, 'name']
+        typeSpec  = self.__widgetSpecs[display, 'overlayType']
+        alphaSpec = self.__widgetSpecs[display, 'alpha']
+        briSpec   = self.__widgetSpecs[display, 'brightness']
+        conSpec   = self.__widgetSpecs[display, 'contrast']
+
 
         # Buttons which toggle overlay
         # info and display panel
+        # This is really hacky. The FSLeyesFrame
+        # setattrs a ToggleControlPanelAction for every
+        # built-in control to the view panel object,
+        # so here we access attributes called
+        # "OverlayDisplayPanel" and "OverlayInfoPanel".
+        # This will hopefully change in the future.
         panelSpec = actions.ToggleActionButton(
-            'toggleDisplayPanel',
+            'OverlayDisplayPanel',
             actionKwargs={'floatPane' : True},
             icon=[icons.findImageFile('gearHighlight24'),
                   icons.findImageFile('gear24')],
-            tooltip=fsltooltips.actions[viewPanel, 'toggleDisplayPanel'])
-
+            tooltip=fsltooltips.actions[viewPanel, 'OverlayDisplayPanel'])
         infoSpec = actions.ToggleActionButton(
-            'toggleOverlayInfo',
+            'OverlayInfoPanel',
             actionKwargs={'floatPane' : True},
             icon=[icons.findImageFile('informationHighlight24'),
                   icons.findImageFile('information24')],
-            tooltip=fsltooltips.actions[viewPanel, 'toggleOverlayInfo'])
+            tooltip=fsltooltips.actions[viewPanel, 'OverlayInfoPanel'])
 
         # Name/overlay type and brightness/contrast
         # are respectively placed together
@@ -314,12 +337,11 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
         """Creates and returns a collection of controls for editing properties
         of the given :class:`.VolumeOpts` instance.
         """
-        rangeSpec      = _TOOLBAR_PROPS[opts, 'displayRange']
-        resetSpec      = _TOOLBAR_PROPS[opts, 'resetDisplayRange']
-        cmapSpec       = _TOOLBAR_PROPS[opts, 'cmap']
-        negCmapSpec    = _TOOLBAR_PROPS[opts, 'negativeCmap']
-        useNegCmapSpec = _TOOLBAR_PROPS[opts, 'useNegativeCmap']
-
+        rangeSpec      = self.__widgetSpecs[opts, 'displayRange']
+        resetSpec      = self.__widgetSpecs[opts, 'resetDisplayRange']
+        cmapSpec       = self.__widgetSpecs[opts, 'cmap']
+        negCmapSpec    = self.__widgetSpecs[opts, 'negativeCmap']
+        useNegCmapSpec = self.__widgetSpecs[opts, 'useNegativeCmap']
 
         cmapPanel = wx.Panel(self)
 
@@ -341,12 +363,38 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
         return tools, nav
 
 
+    def __makeComplexOptsTools(self, opts):
+        """Creates and returns a collection of controls for editing properties
+        of the given :class:`.ComplexOpts` instance.
+        """
+        rangeSpec = self.__widgetSpecs[opts, 'displayRange']
+        resetSpec = self.__widgetSpecs[opts, 'resetDisplayRange']
+        cmapSpec  = self.__widgetSpecs[opts, 'cmap']
+        compSpec  = self.__widgetSpecs[opts, 'component']
+        ccpanel   = wx.Panel(self)
+
+        rangeWidget = props.buildGUI(self,    opts, rangeSpec)
+        resetWidget = props.buildGUI(self,    opts, resetSpec)
+        cmapWidget  = props.buildGUI(ccpanel, opts, cmapSpec)
+        compWidget  = props.buildGUI(ccpanel, opts, compSpec)
+
+        ccsizer = wx.BoxSizer(wx.VERTICAL)
+        ccpanel.SetSizer(ccsizer)
+        ccsizer.Add(cmapWidget, flag=wx.EXPAND)
+        ccsizer.Add(compWidget, flag=wx.EXPAND)
+
+        tools = [resetWidget, rangeWidget, ccpanel]
+        nav   = [resetWidget, rangeWidget, cmapWidget, compWidget]
+
+        return tools, nav
+
+
     def __makeMaskOptsTools(self, opts):
         """Creates and returns a collection of controls for editing properties
         of the given :class:`.MaskOpts` instance.
         """
-        thresSpec  = _TOOLBAR_PROPS[opts, 'threshold']
-        colourSpec = _TOOLBAR_PROPS[opts, 'colour']
+        thresSpec  = self.__widgetSpecs[opts, 'threshold']
+        colourSpec = self.__widgetSpecs[opts, 'colour']
 
         thresWidget  = props.buildGUI(self, opts, thresSpec)
         colourWidget = props.buildGUI(self, opts, colourSpec)
@@ -361,9 +409,9 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
         of the given :class:`.LabelOpts` instance.
         """
 
-        lutSpec     = _TOOLBAR_PROPS[opts, 'lut']
-        outlineSpec = _TOOLBAR_PROPS[opts, 'outline']
-        widthSpec   = _TOOLBAR_PROPS[opts, 'outlineWidth']
+        lutSpec     = self.__widgetSpecs[opts, 'lut']
+        outlineSpec = self.__widgetSpecs[opts, 'outline']
+        widthSpec   = self.__widgetSpecs[opts, 'outlineWidth']
 
         # lut/outline width widgets
         # are on a single panel
@@ -397,9 +445,9 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
         of the given :class:`.VectorOpts` instance.
         """
 
-        modSpec   = _TOOLBAR_PROPS[opts, 'modulateImage']
-        clipSpec  = _TOOLBAR_PROPS[opts, 'clipImage']
-        rangeSpec = _TOOLBAR_PROPS[opts, 'clippingRange']
+        modSpec   = self.__widgetSpecs[opts, 'modulateImage']
+        clipSpec  = self.__widgetSpecs[opts, 'clipImage']
+        rangeSpec = self.__widgetSpecs[opts, 'clippingRange']
 
         panel = wx.Panel(self)
         sizer = wx.GridBagSizer()
@@ -437,7 +485,7 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
         """Creates and returns a collection of controls for editing properties
         of the given :class:`.LineVectorOpts` instance.
         """
-        widthSpec = _TOOLBAR_PROPS[opts, 'lineWidth']
+        widthSpec = self.__widgetSpecs[opts, 'lineWidth']
 
         widget    = props.buildGUI(self, opts, widthSpec)
         lblWidget = self.MakeLabelledTool(
@@ -452,9 +500,9 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
         """Creates and returns a collection of controls for editing properties
         of the given :class:`.MeshOpts` instance.
         """
-        colourSpec  = _TOOLBAR_PROPS[opts, 'colour']
-        outlineSpec = _TOOLBAR_PROPS[opts, 'outline']
-        widthSpec   = _TOOLBAR_PROPS[opts, 'outlineWidth']
+        colourSpec  = self.__widgetSpecs[opts, 'colour']
+        outlineSpec = self.__widgetSpecs[opts, 'outline']
+        widthSpec   = self.__widgetSpecs[opts, 'outlineWidth']
 
         colourWidget  = props.buildGUI(self, opts, colourSpec)
         outlineWidget = props.buildGUI(self, opts, outlineSpec)
@@ -475,8 +523,8 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
         """
         tools, nav = self.__makeMeshOptsTools(opts)
 
-        vertWidget  = _TOOLBAR_PROPS[opts, 'vertexSet']
-        vdataWidget = _TOOLBAR_PROPS[opts, 'vertexData']
+        vertWidget  = self.__widgetSpecs[opts, 'vertexSet']
+        vdataWidget = self.__widgetSpecs[opts, 'vertexData']
 
         panel       = wx.Panel(self)
         sizer       = wx.BoxSizer(wx.VERTICAL)
@@ -504,7 +552,7 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
         """Creates and returns a collection of controls for editing properties
         of the given :class:`.TensorOpts` instance.
         """
-        lightingSpec   = _TOOLBAR_PROPS[opts, 'lighting']
+        lightingSpec   = self.__widgetSpecs[opts, 'lighting']
         lightingWidget = props.buildGUI(self, opts, lightingSpec)
 
         tools, nav = self.__makeVectorOptsTools(opts)
@@ -516,9 +564,8 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
         """Creates and returns a collection of controls for editing properties
         of the given :class:`.SHOpts` instance.
         """
-        sizeSpec = _TOOLBAR_PROPS[opts, 'size']
-        radSpec  = _TOOLBAR_PROPS[opts, 'radiusThreshold']
-
+        sizeSpec = self.__widgetSpecs[opts, 'size']
+        radSpec  = self.__widgetSpecs[opts, 'radiusThreshold']
 
         panel = wx.Panel(self)
         sizer = wx.FlexGridSizer(2, 2, 0, 0)
@@ -546,19 +593,305 @@ class OverlayDisplayToolBar(fsltoolbar.FSLeyesToolBar):
         return tools, nav
 
 
-def _imageLabel(img):
-    """Used to generate labels for the :attr:`.VectorOpts.modulateImage`,
-    :attr:`.VectorOpts.clipImage`, and other :class:`.Image`-based
-    choice properties.
-    """
-    if img is None: return 'None'
-    else:           return img.name
+    def __makeMIPOptsTools(self, opts):
+        """Creates and returns a collection of controls for editing properties
+        of the given :class:`.MIPOpts` instance.
+        """
+        rangeSpec = self.__widgetSpecs[opts, 'displayRange']
+        resetSpec = self.__widgetSpecs[opts, 'resetDisplayRange']
+        cmapSpec  = self.__widgetSpecs[opts, 'cmap']
+
+        cmapPanel = wx.Panel(self)
+
+        rangeWidget = props.buildGUI(self,      opts, rangeSpec)
+        resetWidget = props.buildGUI(self,      opts, resetSpec)
+        cmapWidget  = props.buildGUI(cmapPanel, opts, cmapSpec)
+
+        cmapSizer = wx.BoxSizer(wx.VERTICAL)
+        cmapPanel.SetSizer(cmapSizer)
+        cmapSizer.Add(cmapWidget)
+
+        tools = [resetWidget, rangeWidget, cmapPanel]
+        nav   = [resetWidget, rangeWidget, cmapWidget]
+
+        return tools, nav
 
 
+    def __makeTractogramOptsTools(self, opts):
+        """Creates and returns a collection of controls for editing properties
+        of a :class:`.TractogramOpts` instance.
+        """
 
-def _pathLabel(p):
-    if p is None: return 'None'
-    else:         return op.basename(p)
+        widthSpec  = self.__widgetSpecs[opts, 'lineWidth']
+        resSpec    = self.__widgetSpecs[opts, 'resolution']
+        colourSpec = self.__widgetSpecs[opts, 'colourMode']
+        cmapSpec   = self.__widgetSpecs[opts, 'cmap']
+
+        sliderPanel = wx.Panel(self)
+        colourPanel = wx.Panel(self)
+
+        widthWidget  = props.buildGUI(sliderPanel, opts, widthSpec)
+        resWidget    = props.buildGUI(sliderPanel, opts, resSpec)
+        colourWidget = props.buildGUI(colourPanel, opts, colourSpec)
+        cmapWidget   = props.buildGUI(colourPanel, opts, cmapSpec)
+
+        widthLabel  = wx.StaticText(sliderPanel)
+        resLabel    = wx.StaticText(sliderPanel)
+
+        widthLabel.SetLabel(strings.properties[opts, 'lineWidth'])
+        resLabel  .SetLabel(strings.properties[opts, 'resolution'])
+
+        sliderSizer = wx.FlexGridSizer(2, 2, 0, 0)
+        sliderPanel.SetSizer(sliderSizer)
+        sliderSizer.AddGrowableCol(1)
+        sliderSizer.Add(widthLabel)
+        sliderSizer.Add(widthWidget)
+        sliderSizer.Add(resLabel)
+        sliderSizer.Add(resWidget)
+
+        colourSizer = wx.BoxSizer(wx.VERTICAL)
+        colourPanel.SetSizer(colourSizer)
+        colourSizer.Add(colourWidget, flag=wx.EXPAND, proportion=1)
+        colourSizer.Add(cmapWidget,   flag=wx.EXPAND, proportion=1)
+
+        tools = [sliderPanel, colourPanel]
+        nav   = [widthWidget, resWidget, colourWidget, cmapWidget]
+
+        return tools, nav
+
+
+    def __generateWidgetSpecs(self):
+        """Called by :meth:`__init__`. Creates specifications for the toolbar
+        widgets for all overlay types.
+        """
+
+        def _imageLabel(img):
+            """Used to generate labels for the :attr:`.VectorOpts.modulateImage`,
+            :attr:`.VectorOpts.clipImage`, and other :class:`.Image`-based
+            choice properties.
+            """
+            if img is None: return 'None'
+            else:           return self.displayCtx.getDisplay(img).name
+
+        def _pathLabel(p):
+            if p is None: return 'None'
+            else:         return op.basename(p)
+
+        # TractogramOpts.colourMode
+        def _colourModeLabel(data):
+            if data == 'orientation':
+                return 'Orientation'
+            if data is None:
+                return 'None'
+            elif isinstance(data, fslimage.Image):
+                return self.displayCtx.getDisplay(data).name
+            else:
+                return op.basename(data)
+
+        self.__widgetSpecs = td.TypeDict({
+
+            'Display.name'         : props.Widget(
+                'name',
+                tooltip=_TOOLTIPS['Display.name']),
+            'Display.overlayType'  : props.Widget(
+                'overlayType',
+                tooltip=_TOOLTIPS['Display.overlayType'],
+                labels=strings.choices['Display.overlayType']),
+            'Display.alpha'        : props.Widget(
+                'alpha',
+                spin=False,
+                showLimits=False,
+                tooltip=_TOOLTIPS['Display.alpha']),
+            'Display.brightness'   : props.Widget(
+                'brightness',
+                spin=False,
+                showLimits=False,
+                tooltip=_TOOLTIPS['Display.brightness']),
+            'Display.contrast'     : props.Widget(
+                'contrast',
+                spin=False,
+                showLimits=False,
+                tooltip=_TOOLTIPS['Display.contrast']),
+
+            'VolumeOpts.resetDisplayRange' : actions.ActionButton(
+                'resetDisplayRange',
+                icon=icons.findImageFile('verticalReset32')
+            ),
+
+            'VolumeOpts.displayRange' : props.Widget(
+                'displayRange',
+                slider=False,
+                showLimits=False,
+                spinWidth=10,
+                tooltip=_TOOLTIPS['ColourMapOpts.displayRange'],
+                labels=[strings.choices['ColourMapOpts.displayRange.min'],
+                        strings.choices['ColourMapOpts.displayRange.max']]),
+            'VolumeOpts.cmap' : props.Widget(
+                'cmap',
+                labels=fslcm.getColourMapLabel,
+                tooltip=_TOOLTIPS['VolumeOpts.cmap']),
+            'VolumeOpts.useNegativeCmap' : props.Widget(
+                'useNegativeCmap',
+                icon=[icons.findImageFile('twocmaps24'),
+                      icons.findImageFile('onecmap24')],
+                toggle=True,
+                tooltip=_TOOLTIPS['VolumeOpts.useNegativeCmap']),
+
+            'VolumeOpts.negativeCmap' : props.Widget(
+                'negativeCmap',
+                labels=fslcm.getColourMapLabel,
+                tooltip=_TOOLTIPS['VolumeOpts.negativeCmap'],
+                dependencies=['useNegativeCmap'],
+                enabledWhen=lambda i, unc : unc),
+
+            'ComplexOpts.component' : props.Widget(
+                'component',
+                labels=strings.choices['ComplexOpts.component'],
+                tooltip=_TOOLTIPS['ComplexOpts.component']),
+
+            'MaskOpts.threshold' : props.Widget(
+                'threshold',
+                showLimits=False,
+                spin=True,
+                tooltip=_TOOLTIPS['MaskOpts.threshold'],
+                labels=[strings.choices['MaskOpts.threshold.min'],
+                        strings.choices['MaskOpts.threshold.max']]),
+            'MaskOpts.colour'    : props.Widget(
+                'colour',
+                size=(24, 24),
+                tooltip=_TOOLTIPS['MaskOpts.colour']),
+
+            'LabelOpts.lut'     : props.Widget(
+                'lut',
+                tooltip=_TOOLTIPS['LabelOpts.lut'],
+                labels=lambda l: l.name),
+            'LabelOpts.outline' : props.Widget(
+                'outline',
+                tooltip=_TOOLTIPS['LabelOpts.outline'],
+                icon=[icons.findImageFile('outline24'),
+                      icons.findImageFile('filled24')],
+                toggle=True),
+            'LabelOpts.outlineWidth' : props.Widget(
+                'outlineWidth',
+                tooltip=_TOOLTIPS['LabelOpts.outlineWidth'],
+                showLimits=False,
+                spin=False),
+
+            'MeshOpts.colour'       : props.Widget(
+                'colour',
+                size=(24, 24),
+                tooltip=_TOOLTIPS['MeshOpts.colour']),
+            'MeshOpts.outline'      : props.Widget(
+                'outline',
+                tooltip=_TOOLTIPS['MeshOpts.outline'],
+                icon=[icons.findImageFile('outline24'),
+                      icons.findImageFile('filled24')],
+                toggle=True),
+            'MeshOpts.outlineWidth' : props.Widget(
+                'outlineWidth',
+                showLimits=False,
+                spin=False,
+                tooltip=_TOOLTIPS['MeshOpts.outlineWidth'],
+                enabledWhen=lambda i: i.outline),
+            'MeshOpts.vertexSet' : props.Widget(
+                'vertexSet',
+                labels=_pathLabel,
+                tooltip=_TOOLTIPS['MeshOpts.vertexSet']
+            ),
+            'MeshOpts.vertexData' : props.Widget(
+                'vertexData',
+                labels=_pathLabel,
+                tooltip=_TOOLTIPS['MeshOpts.vertexData']
+            ),
+
+            'VectorOpts.modulateImage' : props.Widget(
+                'modulateImage',
+                labels=_imageLabel,
+                tooltip=_TOOLTIPS['VectorOpts.modulateImage']),
+            'VectorOpts.clipImage' : props.Widget(
+                'clipImage',
+                labels=_imageLabel,
+                tooltip=_TOOLTIPS['VectorOpts.clipImage']),
+            'VectorOpts.clippingRange' : props.Widget(
+                'clippingRange',
+                showLimits=False,
+                slider=True,
+                spin=False,
+                tooltip=_TOOLTIPS['VectorOpts.clippingRange'],
+                labels=[strings.choices['VectorOpts.clippingRange.min'],
+                        strings.choices['VectorOpts.clippingRange.max']],
+                dependencies=['clipImage'],
+                enabledWhen=lambda o, ci: ci is not None),
+
+            'LineVectorOpts.lineWidth' : props.Widget(
+                'lineWidth',
+                showLimits=False,
+                spin=False,
+                tooltip=_TOOLTIPS['LineVectorOpts.lineWidth']),
+
+            'TensorOpts.lighting'      : props.Widget(
+                'lighting',
+                icon=[icons.findImageFile('lightbulbHighlight24'),
+                      icons.findImageFile('lightbulb24')],
+                tooltip=_TOOLTIPS['TensorOpts.lighting']),
+
+            'SHOpts.lighting'        : props.Widget(
+                'lighting',
+                icon=[icons.findImageFile('lightbulbHighlight24'),
+                      icons.findImageFile('lightbulb24')],
+                tooltip=_TOOLTIPS['SHOpts.lighting']),
+            'SHOpts.size'            : props.Widget(
+                'size',
+                showLimits=False,
+                slider=True,
+                spin=False,
+                tooltip=_TOOLTIPS['SHOpts.size']),
+            'SHOpts.radiusThreshold' : props.Widget(
+                'radiusThreshold',
+                slider=True,
+                spin=False,
+                showLimits=False,
+                tooltip=_TOOLTIPS['SHOpts.radiusThreshold']),
+
+            'MIPOpts.resetDisplayRange' : actions.ActionButton(
+                'resetDisplayRange',
+                icon=icons.findImageFile('verticalReset32')
+            ),
+
+            'MIPOpts.displayRange' : props.Widget(
+                'displayRange',
+                slider=False,
+                showLimits=False,
+                spinWidth=10,
+                tooltip=_TOOLTIPS['ColourMapOpts.displayRange'],
+                labels=[strings.choices['ColourMapOpts.displayRange.min'],
+                        strings.choices['ColourMapOpts.displayRange.max']]),
+            'MIPOpts.cmap' : props.Widget(
+                'cmap',
+                labels=fslcm.getColourMapLabel,
+                tooltip=_TOOLTIPS['VolumeOpts.cmap']),
+
+            'TractogramOpts.lineWidth' : props.Widget(
+                'lineWidth',
+                showLimits=False,
+                slider=True,
+                spin=False,
+                tooltip=_TOOLTIPS['TractogramOpts.lineWidth']),
+            'TractogramOpts.resolution' : props.Widget(
+                'resolution',
+                showLimits=False,
+                slider=True,
+                spin=False,
+                tooltip=_TOOLTIPS['TractogramOpts.resolution']),
+            'TractogramOpts.colourMode' : props.Widget(
+                'colourMode',
+                labels=_colourModeLabel,
+                tooltip=_TOOLTIPS['TractogramOpts.colourMode']),
+            'TractogramOpts.cmap' : props.Widget(
+                'cmap',
+                labels=fslcm.getColourMapLabel,
+                tooltip=_TOOLTIPS['VolumeOpts.cmap']),
+        })
 
 
 _TOOLTIPS = td.TypeDict({
@@ -578,10 +911,13 @@ _TOOLTIPS = td.TypeDict({
     'VolumeOpts.useNegativeCmap'   :
     fsltooltips.properties['ColourMapOpts.useNegativeCmap'],
 
+    'ComplexOpts.component' : fsltooltips.properties['ComplexOpts.component'],
+
     'MaskOpts.threshold' : fsltooltips.properties['MaskOpts.threshold'],
     'MaskOpts.colour'    : fsltooltips.properties['MaskOpts.colour'],
 
     'LabelOpts.lut'          : fsltooltips.properties['LabelOpts.lut'],
+
     'LabelOpts.outline'      : fsltooltips.properties['LabelOpts.outline'],
     'LabelOpts.outlineWidth' : fsltooltips.properties['LabelOpts.'
                                                       'outlineWidth'],
@@ -610,173 +946,15 @@ _TOOLTIPS = td.TypeDict({
     'SHOpts.size'            : fsltooltips.properties['SHOpts.size'],
     'SHOpts.radiusThreshold' : fsltooltips.properties['SHOpts.'
                                                       'radiusThreshold'],
+
+    'TractogramOpts.lineWidth' :
+    fsltooltips.properties['TractogramOpts.lineWidth'],
+    'TractogramOpts.resolution' :
+    fsltooltips.properties['TractogramOpts.resolution'],
+    'TractogramOpts.colourMode' :
+    fsltooltips.properties['TractogramOpts.colourMode'],
 })
 """This dictionary contains tooltips for :class:`.Display` and
-:class:`.DisplayOpts` properties. It is referenced in the
-:attr:`_TOOLBAR_PROPS` dictionary definition.
-"""
-
-
-_TOOLBAR_PROPS = td.TypeDict({
-
-    'Display.name'         : props.Widget(
-        'name',
-        tooltip=_TOOLTIPS['Display.name']),
-    'Display.overlayType'  : props.Widget(
-        'overlayType',
-        tooltip=_TOOLTIPS['Display.overlayType'],
-        labels=strings.choices['Display.overlayType']),
-    'Display.alpha'        : props.Widget(
-        'alpha',
-        spin=False,
-        showLimits=False,
-        tooltip=_TOOLTIPS['Display.alpha']),
-    'Display.brightness'   : props.Widget(
-        'brightness',
-        spin=False,
-        showLimits=False,
-        tooltip=_TOOLTIPS['Display.brightness']),
-    'Display.contrast'     : props.Widget(
-        'contrast',
-        spin=False,
-        showLimits=False,
-        tooltip=_TOOLTIPS['Display.contrast']),
-
-    'VolumeOpts.resetDisplayRange' : actions.ActionButton(
-        'resetDisplayRange',
-        icon=icons.findImageFile('verticalReset32')
-    ),
-
-    'VolumeOpts.displayRange' : props.Widget(
-        'displayRange',
-        slider=False,
-        showLimits=False,
-        spinWidth=10,
-        tooltip=_TOOLTIPS['ColourMapOpts.displayRange'],
-        labels=[strings.choices['ColourMapOpts.displayRange.min'],
-                strings.choices['ColourMapOpts.displayRange.max']]),
-    'VolumeOpts.cmap' : props.Widget(
-        'cmap',
-        labels=fslcm.getColourMapLabel,
-        tooltip=_TOOLTIPS['VolumeOpts.cmap']),
-    'VolumeOpts.useNegativeCmap' : props.Widget(
-        'useNegativeCmap',
-        icon=[icons.findImageFile('twocmaps24'),
-              icons.findImageFile('onecmap24')],
-        toggle=True,
-        tooltip=_TOOLTIPS['VolumeOpts.useNegativeCmap']),
-
-    'VolumeOpts.negativeCmap' : props.Widget(
-        'negativeCmap',
-        labels=fslcm.getColourMapLabel,
-        tooltip=_TOOLTIPS['VolumeOpts.negativeCmap'],
-        dependencies=['useNegativeCmap'],
-        enabledWhen=lambda i, unc : unc),
-
-    'MaskOpts.threshold' : props.Widget(
-        'threshold',
-        showLimits=False,
-        spin=True,
-        tooltip=_TOOLTIPS['MaskOpts.threshold'],
-        labels=[strings.choices['MaskOpts.threshold.min'],
-                strings.choices['MaskOpts.threshold.max']]),
-    'MaskOpts.colour'    : props.Widget(
-        'colour',
-        size=(24, 24),
-        tooltip=_TOOLTIPS['MaskOpts.colour']),
-
-    'LabelOpts.lut'     : props.Widget(
-        'lut',
-        tooltip=_TOOLTIPS['LabelOpts.lut'],
-        labels=lambda l: l.name),
-    'LabelOpts.outline' : props.Widget(
-        'outline',
-        tooltip=_TOOLTIPS['LabelOpts.outline'],
-        icon=[icons.findImageFile('outline24'),
-              icons.findImageFile('filled24')],
-        toggle=True),
-    'LabelOpts.outlineWidth' : props.Widget(
-        'outlineWidth',
-        tooltip=_TOOLTIPS['LabelOpts.outlineWidth'],
-        showLimits=False,
-        spin=False),
-
-    'MeshOpts.colour'       : props.Widget(
-        'colour',
-        size=(24, 24),
-        tooltip=_TOOLTIPS['MeshOpts.colour']),
-    'MeshOpts.outline'      : props.Widget(
-        'outline',
-        tooltip=_TOOLTIPS['MeshOpts.outline'],
-        icon=[icons.findImageFile('outline24'),
-              icons.findImageFile('filled24')],
-        toggle=True),
-    'MeshOpts.outlineWidth' : props.Widget(
-        'outlineWidth',
-        showLimits=False,
-        spin=False,
-        tooltip=_TOOLTIPS['MeshOpts.outlineWidth'],
-        enabledWhen=lambda i: i.outline),
-    'MeshOpts.vertexSet' : props.Widget(
-        'vertexSet',
-        labels=_pathLabel,
-        tooltip=_TOOLTIPS['MeshOpts.vertexSet']
-    ),
-    'MeshOpts.vertexData' : props.Widget(
-        'vertexData',
-        labels=_pathLabel,
-        tooltip=_TOOLTIPS['MeshOpts.vertexData']
-    ),
-
-    'VectorOpts.modulateImage' : props.Widget(
-        'modulateImage',
-        labels=_imageLabel,
-        tooltip=_TOOLTIPS['VectorOpts.modulateImage']),
-    'VectorOpts.clipImage' : props.Widget(
-        'clipImage',
-        labels=_imageLabel,
-        tooltip=_TOOLTIPS['VectorOpts.clipImage']),
-    'VectorOpts.clippingRange' : props.Widget(
-        'clippingRange',
-        showLimits=False,
-        slider=True,
-        spin=False,
-        tooltip=_TOOLTIPS['VectorOpts.clippingRange'],
-        labels=[strings.choices['VectorOpts.clippingRange.min'],
-                strings.choices['VectorOpts.clippingRange.max']],
-        dependencies=['clipImage'],
-        enabledWhen=lambda o, ci: ci is not None),
-
-    'LineVectorOpts.lineWidth' : props.Widget(
-        'lineWidth',
-        showLimits=False,
-        spin=False,
-        tooltip=_TOOLTIPS['LineVectorOpts.lineWidth']),
-
-    'TensorOpts.lighting'      : props.Widget(
-        'lighting',
-        icon=[icons.findImageFile('lightbulbHighlight24'),
-              icons.findImageFile('lightbulb24')],
-        tooltip=_TOOLTIPS['TensorOpts.lighting']),
-
-    'SHOpts.lighting'        : props.Widget(
-        'lighting',
-        icon=[icons.findImageFile('lightbulbHighlight24'),
-              icons.findImageFile('lightbulb24')],
-        tooltip=_TOOLTIPS['SHOpts.lighting']),
-    'SHOpts.size'            : props.Widget(
-        'size',
-        showLimits=False,
-        slider=True,
-        spin=False,
-        tooltip=_TOOLTIPS['SHOpts.size']),
-    'SHOpts.radiusThreshold' : props.Widget(
-        'radiusThreshold',
-        slider=True,
-        spin=False,
-        showLimits=False,
-        tooltip=_TOOLTIPS['SHOpts.radiusThreshold']),
-})
-"""This dictionary defines specifications for all controls shown on an
-:class:`OverlayDisplayToolBar`.
+:class:`.DisplayOpts` properties. It is used by the
+:meth:`OverlayDisplayToolBar.__generateWidgetSpecs` method.
 """
