@@ -13,6 +13,7 @@ import copy
 import logging
 
 import fsleyes_props      as props
+import fsleyes.gl         as fslgl
 import fsleyes.colourmaps as fslcm
 
 from . import canvasopts
@@ -32,10 +33,10 @@ class SceneOpts(props.HasProperties):
 
 
     showCursor      = copy.copy(canvasopts.SliceCanvasOpts.showCursor)
+    cursorWidth     = copy.copy(canvasopts.SliceCanvasOpts.cursorWidth)
     zoom            = copy.copy(canvasopts.SliceCanvasOpts.zoom)
     bgColour        = copy.copy(canvasopts.SliceCanvasOpts.bgColour)
     cursorColour    = copy.copy(canvasopts.SliceCanvasOpts.cursorColour)
-    renderMode      = copy.copy(canvasopts.SliceCanvasOpts.renderMode)
     highDpi         = copy.copy(canvasopts.SliceCanvasOpts.highDpi)
 
 
@@ -66,34 +67,51 @@ class SceneOpts(props.HasProperties):
     """
 
 
-    # NOTE: If you change the maximum performance value,
-    #       make sure you update all references to
-    #       performance because, for example, the
-    #       OrthoEditProfile does numerical comparisons
-    #       to it.
-    performance = props.Choice((1, 2, 3), default=3, allowStr=True)
-    """User controllable performance setting.
+    colourBarSize = props.Percentage(default=100)
+    """Size of the major axis of the colour bar, as a proportion of the
+    available space.
+    """
 
-    This property is linked to the :attr:`renderMode` property. Setting this
-    property to a low value will result in faster rendering time, at the cost
-    of increased memory usage and poorer rendering quality.
 
-    See the :meth:`__onPerformanceChange` method.
+    labelSize = props.Int(minval=4, maxval=96, default=12, clamped=True)
+    """Font size used for any labels drawn on the canvas, including
+    orthographic labels, and colour bar labels.
+    """
+
+
+    movieSyncRefresh = props.Boolean(default=True)
+    """Whether, when in movie mode, to synchronise the refresh for GL
+    canvases. This is not possible in some platforms/environments. See
+    :attr:`.CanvasPanel.movieSyncRefresh`.
     """
 
 
     def __init__(self, panel):
-        """Create a ``SceneOpts`` instance.
-
-        This method simply links the :attr:`performance` property to the
-        :attr:`renderMode` property.
-        """
+        """Create a ``SceneOpts`` instance. """
 
         self.__panel = panel
         self.__name  = '{}_{}'.format(type(self).__name__, id(self))
-        self.addListener('performance', self.__name, self._onPerformanceChange)
-        self.addListener('bgColour',    self.__name, self.__onBgColourChange)
-        self._onPerformanceChange()
+
+        self.movieSyncRefresh = self.defaultMovieSyncRefresh
+        self.addListener('bgColour', self.__name, self.__onBgColourChange)
+
+
+    @property
+    def defaultMovieSyncRefresh(self):
+        """In movie mode, the canvas refreshes are performed by the
+        __syncMovieRefresh or __unsyncMovieRefresh methods of the CanvasPanel
+        class. Some platforms/GL drivers/environments seem to have a problem
+        with separate renders/buffer swaps, so we have to use a shitty
+        unsynchronised update routine.
+
+        These heuristics are not perfect - the movieSyncRefresh property can
+        therefore be overridden by the user.
+        """
+        renderer        = fslgl.GL_RENDERER.lower()
+        unsyncRenderers = ['gallium', 'mesa dri intel(r)']
+        unsync          = any([r in renderer for r in unsyncRenderers])
+
+        return not unsync
 
 
     @property
@@ -104,17 +122,6 @@ class SceneOpts(props.HasProperties):
         return self.__panel
 
 
-    def _onPerformanceChange(self, *a):
-        """Called when the :attr:`performance` property changes.
-
-        This method must be overridden by sub-classes to change the values of
-        the :attr:`renderMode` property according to the new performance
-        setting.
-        """
-        raise NotImplementedError('The _onPerformanceChange method must'
-                                  'be implemented by sub-classes')
-
-
     def __onBgColourChange(self, *a):
         """Called when the background colour changes. Updates the
         :attr:`fgColour` to a complementary colour.
