@@ -362,6 +362,13 @@ class FSLeyesFrame(wx.Frame):
         return list(self.__viewPanels)
 
 
+    def getView(self, panelType):
+        """Return a list of all :class:`.ViewPanel` instances of the specified
+        type.
+        """
+        return [v for v in self.__viewPanels if isinstance(v, panelType)]
+
+
     @property
     def focusedViewPanel(self):
         """Returns the :class:`.ViewPanel` which currently has focus,
@@ -506,6 +513,7 @@ class FSLeyesFrame(wx.Frame):
         childDC = displaycontext.DisplayContext(
             self.__overlayList,
             parent=self.__displayCtx,
+            displayType=panelCls.displayType(),
             **kwargs)
 
         panel = panelCls(self.__mainPanel, self.__overlayList, childDC, self)
