@@ -9,11 +9,9 @@ types in an OpenGL 1.4 compatible manner.
 """
 
 from . import glvolume_funcs      # noqa
+from . import glrgbvolume_funcs   # noqa
 from . import glrgbvector_funcs   # noqa
 from . import gllinevector_funcs  # noqa
 from . import glmesh_funcs        # noqa
 from . import glmask_funcs        # noqa
 from . import gllabel_funcs       # noqa
-
-gltensor_funcs = None
-glsh_funcs     = None
