@@ -29,8 +29,11 @@ This module also provides a few functions, most importantly
    createGLObject
 """
 
+
+import contextlib
 import logging
 
+import fsl.utils.idle     as idle
 import fsl.utils.notifier as notifier
 
 
@@ -43,6 +46,8 @@ def getGLObjectType(overlayType):
     """
 
     from . import glvolume
+    from . import glrgbvolume
+    from . import glcomplex
     from . import glmask
     from . import glrgbvector
     from . import gllinevector
@@ -50,6 +55,8 @@ def getGLObjectType(overlayType):
     from . import gllabel
     from . import gltensor
     from . import glsh
+    from . import glmip
+    from . import gltractogram
 
     typeMap = {
         'volume'     : glvolume    .GLVolume,
@@ -59,13 +66,17 @@ def getGLObjectType(overlayType):
         'mesh'       : glmesh      .GLMesh,
         'label'      : gllabel     .GLLabel,
         'tensor'     : gltensor    .GLTensor,
-        'sh'         : glsh        .GLSH
+        'sh'         : glsh        .GLSH,
+        'mip'        : glmip       .GLMIP,
+        'rgb'        : glrgbvolume .GLRGBVolume,
+        'complex'    : glcomplex   .GLComplex,
+        'tractogram' : gltractogram.GLTractogram
     }
 
     return typeMap.get(overlayType, None)
 
 
-def createGLObject(overlay, overlayList, displayCtx, canvas, threedee=False):
+def createGLObject(overlay, overlayList, displayCtx, threedee=False):
     """Create :class:`GLObject` instance for the given overlay, as specified
     by the :attr:`.Display.overlayType` property.
 
@@ -75,8 +86,6 @@ def createGLObject(overlay, overlayList, displayCtx, canvas, threedee=False):
 
     :arg displayCtx:  The :class:`.DisplayContext` managing the scene.
 
-    :arg canvas:      The canvas which will be displaying this ``GLObject``.
-
     :arg threedee:    If ``True``, the ``GLObject`` will be configured for
                       3D rendering. Otherwise it will be configured for 2D
                       slice-based rendering.
@@ -86,7 +95,7 @@ def createGLObject(overlay, overlayList, displayCtx, canvas, threedee=False):
     ctr     = getGLObjectType(display.overlayType)
 
     if ctr is not None:
-        return ctr(overlay, overlayList, displayCtx, canvas, threedee)
+        return ctr(overlay, overlayList, displayCtx, threedee)
     else:
         return None
 
@@ -115,11 +124,6 @@ class GLObject(notifier.Notifier):
       - ``displayCtx``: The :class:`.DisplayContext` managing the scene
                         that this ``GLObject`` is a part of.
 
-      - ``canvas``:     The canvas which is displaying this ``GLObject``.
-                        Could be a :class:`.SliceCanvas`, a
-                        :class:`.LightBoxCanvas`, a :class:`.Scene3DCanvas`,
-                        or some future not-yet-created canvas.
-
       - ``threedee``:   A boolean flag indicating whether this ``GLObject``
                         is configured for 2D or 3D rendering.
 
@@ -196,7 +200,7 @@ class GLObject(notifier.Notifier):
     """
 
 
-    def __init__(self, overlay, overlayList, displayCtx, canvas, threedee):
+    def __init__(self, overlay, overlayList, displayCtx, threedee):
         """Create a :class:`GLObject`.  The constructor adds one attribute
         to this instance, ``name``, which is simply a unique name for this
         instance.
@@ -211,8 +215,6 @@ class GLObject(notifier.Notifier):
 
         :arg displayCtx:  The ``DisplayContext`` managing the scene
 
-        :arg canvas:      The canvas that is displaying this ``GLObject``.
-
         :arg threedee:    Whether this ``GLObject`` is to be used for 2D or 3D
                           rendering.
         """
@@ -222,7 +224,6 @@ class GLObject(notifier.Notifier):
         self.__overlay     = overlay
         self.__overlayList = overlayList
         self.__displayCtx  = displayCtx
-        self.__canvas      = canvas
         self.__display     = None
         self.__opts        = None
 
@@ -253,12 +254,6 @@ class GLObject(notifier.Notifier):
         return self.__overlay
 
 
-    @property
-    def canvas(self):
-        """The canvas which is drawing this ``GLObject``."""
-        return self.__canvas
-
-
     @property
     def display(self):
         """The :class:`.Display` instance containing overlay display
@@ -308,6 +303,14 @@ class GLObject(notifier.Notifier):
                                   'implemented by GLObject subclasses')
 
 
+    def notifyWhen(self, condition):
+        """Wrapper around :meth:`.Notifier.notify` which schedule the
+        ``notify`` call to take place when ``condition() is True``.
+        """
+        name = f'{self.name}_notifyWhen'
+        idle.idleWhen(self.notify, condition, name=name, skipIfQueued=True)
+
+
     def getDisplayBounds(self):
         """This method must calculate and return a bounding box, in the
         display coordinate system, which contains the entire ``GLObject``.
@@ -368,6 +371,7 @@ class GLObject(notifier.Notifier):
         self.__displayCtx = None
 
 
+    @property
     def destroyed(self):
         """This method may be called to test whether a call has been made to
         :meth:`destroy`.
@@ -378,62 +382,54 @@ class GLObject(notifier.Notifier):
         raise NotImplementedError()
 
 
-    def preDraw(self, xform=None, bbox=None):
+    def preDraw(self):
         """This method is called at the start of a draw routine.
 
         It should perform any initialisation which is required before one or
         more calls to the :meth:`draw2D`/:meth:`draw3D` methods are made, such
         as binding and configuring textures.
-
-        See :meth:`draw2D` for details on the ``xform`` and ``bbox``
-        arguments.  They are only guaranteed to be passed to the ``preDraw``
-        method in scenarios where only a single call to ``draw2D``
-        or``draw3D`` is made between calls to ``preDraw`` and ``postDraw``.
         """
-        raise NotImplementedError()
 
 
-    def draw2D(self, zpos, axes, xform=None, bbox=None):
+    def draw2D(self, canvas, zpos, axes, xform=None):
         """This method is called on ``GLObject`` instances which are
         configured for 2D rendering. It should draw a view of this
         ``GLObject`` - a 2D slice at the given Z location, which specifies
         the position along the screen depth axis.
 
-        :arg zpos:  Position along Z axis to draw.
+        :arg canvas: The canvas which is displaying this ``GLObject``.
+                     Could be a :class:`.SliceCanvas`, a
+                     :class:`.LightBoxCanvas`, a :class:`.Scene3DCanvas`,
+                     or some future not-yet-created canvas.
 
-        :arg axes:  Tuple containing the ``(x, y, z)`` axes in the
-                    display coordinate system The ``x`` and ``y`` axes
-                    correspond to the horizontal and vertical display axes
-                    respectively, and the ``z`` to the depth.
+        :arg zpos:   Position along Z axis to draw.
 
-        :arg xform: If provided, it must be applied to the model view
-                    transformation before drawing.
+        :arg axes:   Tuple containing the ``(x, y, z)`` axes in the
+                     display coordinate system The ``x`` and ``y`` axes
+                     correspond to the horizontal and vertical display axes
+                     respectively, and the ``z`` to the depth.
 
-        :arg bbox:  If provided, defines the bounding box, in the display
-                    coordinate system, which is to be displayed. Can be used
-                    as a performance hint (i.e. to limit the number of things
-                    that are rendered).
+        :arg xform:  If provided, it must be applied to the model view
+                     transformation before drawing.
         """
-        raise NotImplementedError()
 
 
-    def draw3D(self, xform=None, bbox=None):
+    def draw3D(self, canvas, xform=None):
         """This method is called on ``GLObject`` instances which are
         configured for 3D rendering. It should draw a 3D view of this
         ``GLObject``.
 
-        :arg xform: If provided, it must be applied to the model view
-                    transformation before drawing.
+        :arg canvas: The canvas which is displaying this ``GLObject``.
+                     Could be a :class:`.SliceCanvas`, a
+                     :class:`.LightBoxCanvas`, a :class:`.Scene3DCanvas`,
+                     or some future not-yet-created canvas.
 
-        :arg bbox:  If provided, defines the bounding box, in the display
-                    coordinate system, which is to be displayed. Can be used
-                    as a performance hint (i.e. to limit the number of things
-                    that are rendered).
+        :arg xform:  If provided, it must be applied to the model view
+                     transformation before drawing.
         """
-        raise NotImplementedError()
 
 
-    def drawAll(self, axes, zposes, xforms):
+    def drawAll(self, canvas, axes, zposes, xforms):
         """This is a convenience method for 2D lightboxD canvases, where
         multple 2D slices at different depths are drawn alongside each other.
 
@@ -450,20 +446,16 @@ class GLObject(notifier.Notifier):
         by combining the draws.
         """
         for (zpos, xform) in zip(zposes, xforms):
-            self.draw2D(zpos, axes, xform)
+            self.draw2D(canvas, zpos, axes, xform)
 
 
-    def postDraw(self, xform=None, bbox=None):
+    def postDraw(self):
         """This method is called after the :meth:`draw2D`/:meth:`draw3D`
         methods have been called one or more times.
 
         It should perform any necessary cleaning up, such as unbinding
         textures.
-
-        See the :meth:`draw2D` method for details on the ``xform`` and
-        ``bbox`` arguments.
         """
-        raise NotImplementedError()
 
 
 class GLSimpleObject(GLObject):
@@ -482,13 +474,14 @@ class GLSimpleObject(GLObject):
     .. note:: The :attr:`GLObject.overlay`, :attr:`GLObject.display`,
               :attr:`GLObject.opts`, :attr:`GLObject.canvas`,
               :attr:`GLObject.overlayList` and :attr:`GLObject.displayCtx`
-              properties of a ``GLSimpleObject`` are all set to ``None``.
+              properties of a ``GLSimpleObject`` will all potentially be
+              set to ``None``.
     """
 
 
     def __init__(self, threedee):
         """Create a ``GLSimpleObject``. """
-        GLObject.__init__(self, None, None, None, None, threedee)
+        GLObject.__init__(self, None, None, None, threedee)
         self.__destroyed = False
 
 
@@ -503,6 +496,7 @@ class GLSimpleObject(GLObject):
         self.__destroyed = True
 
 
+    @property
     def destroyed(self):
         """Overrides :meth:`GLObject.destroy`. Returns ``True`` if
         :meth:`destroy` hs been called, ``False`` otherwise.
@@ -510,11 +504,9 @@ class GLSimpleObject(GLObject):
         return self.__destroyed
 
 
-    def preDraw(self, *args, **kwargs):
+    def preDraw(self):
         """Overrides :meth:`GLObject.preDraw`. Does nothing. """
-        pass
 
 
-    def postDraw(self, *args, **kwargs):
+    def postDraw(self):
         """Overrides :meth:`GLObject.postDraw`. Does nothing. """
-        pass
