@@ -43,35 +43,33 @@ class GLRGBVector(glvector.GLVector):
     These modules are assumed to provide the following functions:
 
 
-    ========================================== ===============================
-    ``init(GLRGBVector)``                      Perform any necessary
-                                               initialisation.
-    ``destroy(GLRGBVector)``                   Perform any necessary clean up.
-    ``compileShaders(GLRGBVector)``            Compiles vertex/fragment
-                                               shaders.
-    ``updateShaderState(GLRGBVector)``         Updates vertex/fragment
-                                               shaders.
-    ``preDraw(GLRGBVector, xform, bbox)``      Prepare the GL state for
-                                               drawing.
-    ``draw2D(GLRGBVector, zpos, xform, bbox)`` Draw the slice specified by
-                                               ``zpos``.
-    ``draw3D(GLRGBVector, zpos, xform)``       Draw the volume in 3D
-    ``drawAll(GLRGBVector, zposes, xforms)``   Draw all slices specified by
-                                               ``zposes``.
-    ``postDraw(GLRGBVector, xform, bbox)``     Clean up the GL state after
-                                               drawing.
-    ========================================== ===============================
+    ======================================== ===============================
+    ``init(GLRGBVector)``                    Perform any necessary
+                                             initialisation.
+    ``destroy(GLRGBVector)``                 Perform any necessary clean up.
+    ``compileShaders(GLRGBVector)``          Compiles vertex/fragment
+                                             shaders.
+    ``updateShaderState(GLRGBVector)``       Updates vertex/fragment
+                                             shaders.
+    ``preDraw(GLRGBVector)``                 Prepare the GL state for
+                                             drawing.
+    ``draw2D(GLRGBVector, zpos, xform)``     Draw the slice specified by
+                                             ``zpos``.
+    ``drawAll(GLRGBVector, zposes, xforms)`` Draw all slices specified by
+                                             ``zposes``.
+    ``postDraw(GLRGBVector)``                Clean up the GL state after
+                                             drawing.
+    ======================================== ===============================
     """
 
 
-    def __init__(self, image, overlayList, displayCtx, canvas, threedee):
+    def __init__(self, image, overlayList, displayCtx, threedee):
         """Create a ``GLRGBVector``.
 
         :arg image:       An :class:`.Image` or :class:`.DTIFitTensor`
                           instance.
         :arg overlayList: The :class:`.OverlayList`
         :arg displayCtx:  The :class:`.DisplayContext` managing the scene.
-        :arg canvas:      The canvas doing the drawing.
         :arg threedee:    2D or 3D rendering
         """
 
@@ -82,8 +80,18 @@ class GLRGBVector(glvector.GLVector):
         else:                                      vecImage = image
 
         def prefilter(data):
-            # make absolute and scale to
-            # unit length if required
+
+            # Vector images stored as RGB24 data
+            # type are assumed to map from [0, 255]
+            # to [-1, 1], so cannot be normalised
+            if vecImage.nvals > 1:
+                return data
+
+            # make absolute, and scale to unit
+            # length if required. We must make
+            # the data absolute, otherwise we
+            # cannot perform interpolation on
+            # the texture when displaying it.
             data = np.abs(data)
             if self.opts.unitLength:
                 with np.errstate(invalid='ignore'):
@@ -97,7 +105,7 @@ class GLRGBVector(glvector.GLVector):
             return data
 
         def prefilterRange(dmin, dmax):
-            if self.unitLength:
+            if self.opts.unitLength:
                 return 0, 1
             else:
                 return max((0, dmin)), max((abs(dmin), abs(dmax)))
@@ -106,7 +114,6 @@ class GLRGBVector(glvector.GLVector):
                                    image,
                                    overlayList,
                                    displayCtx,
-                                   canvas,
                                    threedee,
                                    prefilter=prefilter,
                                    prefilterRange=prefilterRange,
@@ -122,7 +129,6 @@ class GLRGBVector(glvector.GLVector):
                               self.__unitLengthChanged)
 
 
-
     def destroy(self):
         """Must be called when this ``GLRGBVector`` is no longer needed.
         Removes some property listeners from the :class:`.RGBVectorOpts`
@@ -147,7 +153,7 @@ class GLRGBVector(glvector.GLVector):
         glvector.GLVector.refreshImageTexture(self, interp)
 
 
-    def refreshAuxTexture(self, which):
+    def registerAuxImage(self, which, image, onReady=None):
         """Overrides :meth:`.GLVector.refreshAuxTexture`. Calls the base
         class implementation.
         """
@@ -156,7 +162,8 @@ class GLRGBVector(glvector.GLVector):
         if opts.interpolation == 'none': interp = gl.GL_NEAREST
         else:                            interp = gl.GL_LINEAR
 
-        glvector.GLVector.refreshAuxTexture(self, which, interp)
+        glvector.GLVector.registerAuxImage(
+            self, which, image, onReady, interp=interp)
 
 
     def __interpChanged(self, *a):
@@ -174,6 +181,7 @@ class GLRGBVector(glvector.GLVector):
         self.colourTexture  .set(interp=interp)
         self.asyncUpdateShaderState(alwaysNotify=True)
 
+
     def __unitLengthChanged(self, *a):
         """Called when :attr:`.RGBVectorOpts.unitLength` changes. Refreshes
         the texture data.
@@ -195,30 +203,27 @@ class GLRGBVector(glvector.GLVector):
         return fslgl.glrgbvector_funcs.updateShaderState(self)
 
 
-    def preDraw(self, xform=None, bbox=None):
+    def preDraw(self):
         """Overrides :meth:`.GLVector.preDraw`. Calls the base class
         implementation, and the OpenGL version-specific ``preDraw`` function.
         """
-        glvector.GLVector.preDraw(self, xform, bbox)
-        fslgl.glrgbvector_funcs.preDraw(self, xform, bbox)
+        glvector.GLVector.preDraw(self)
+        fslgl.glrgbvector_funcs.preDraw(self)
 
 
-    def draw2D(self, *args, **kwargs):
+    def draw2D(self, canvas, *args, **kwargs):
         """Overrides :meth:`.GLVector.draw2D`. Calls the OpenGL
         version-specific ``draw2D`` function.
         """
         with glroutines.enabled((gl.GL_CULL_FACE)):
             gl.glPolygonMode(gl.GL_FRONT_AND_BACK, gl.GL_FILL)
             gl.glCullFace(gl.GL_BACK)
-            gl.glFrontFace(self.frontFace())
-            fslgl.glrgbvector_funcs.draw2D(self, *args, **kwargs)
+            gl.glFrontFace(self.frontFace(canvas))
+            fslgl.glrgbvector_funcs.draw2D(self, canvas, *args, **kwargs)
 
 
     def draw3D(self, *args, **kwargs):
-        """Overrides :meth:`.GLVector.draw3D`. Calls the OpenGL
-        version-specific ``draw3D`` function.
-        """
-        fslgl.glrgbvector_funcs.draw3D(self, *args, **kwargs)
+        """Does nothing. """
 
 
     def drawAll(self, *args, **kwargs):
@@ -228,10 +233,10 @@ class GLRGBVector(glvector.GLVector):
         fslgl.glrgbvector_funcs.drawAll(self, *args, **kwargs)
 
 
-    def postDraw(self, xform=None, bbox=None):
+    def postDraw(self):
         """Overrides :meth:`.GLVector.postDraw`. Calls the base class
         implementation, and the OpenGL version-specific ``postDraw``
         function.
         """
-        glvector.GLVector.postDraw(self, xform, bbox)
-        fslgl.glrgbvector_funcs.postDraw(self, xform, bbox)
+        glvector.GLVector.postDraw(self)
+        fslgl.glrgbvector_funcs.postDraw(self)
