@@ -62,7 +62,7 @@ class GLVolume(glimageobject.GLImageObject):
     ``updateShaderState(GLVolume)``       Updates the shader program states
                                           when display parameters are changed.
 
-    ``preDraw(GLVolume, xform, bbox)``    Initialise the GL state, ready for
+    ``preDraw(GLVolume)``                 Initialise the GL state, ready for
                                           drawing.
 
     ``draw2D(GLVolume, zpos, xform)``     Draw a slice of the image at the
@@ -71,7 +71,7 @@ class GLVolume(glimageobject.GLImageObject):
                                           transformation on the vertex
                                           coordinates.
 
-    ``draw3D(GLVolume, xform, bbox)``     Draw the image in 3D. If ``xform``
+    ``draw3D(GLVolume, xform)``           Draw the image in 3D. If ``xform``
                                           is not ``None``, it must be applied
                                           as a transformation on the vertex
                                           coordinates.
@@ -80,7 +80,7 @@ class GLVolume(glimageobject.GLImageObject):
                                           specified ``zposes``, applying the
                                           corresponding ``xforms`` to each.
 
-    ``postDraw(GLVolume, xform, bbox)``   Clear the GL state after drawing.
+    ``postDraw(GLVolume)``                Clear the GL state after drawing.
     ===================================== =====================================
 
 
@@ -186,7 +186,7 @@ class GLVolume(glimageobject.GLImageObject):
     """
 
 
-    def __init__(self, image, overlayList, displayCtx, canvas, threedee):
+    def __init__(self, image, overlayList, displayCtx, threedee):
         """Create a ``GLVolume`` object.
 
         :arg image:       An :class:`.Image` object.
@@ -196,8 +196,6 @@ class GLVolume(glimageobject.GLImageObject):
         :arg displayCtx:  The :class:`.DisplayContext` object managing the
                           scene.
 
-        :arg canvas:      The canvas doing the drawing.
-
         :arg threedee:    Set up for 2D or 3D rendering.
         """
 
@@ -205,7 +203,6 @@ class GLVolume(glimageobject.GLImageObject):
                                              image,
                                              overlayList,
                                              displayCtx,
-                                             canvas,
                                              threedee)
 
         # Create an image texture, clip texture, and a colour map texture
@@ -224,7 +221,7 @@ class GLVolume(glimageobject.GLImageObject):
 
         # aux texture manager takes care
         # of clip+modulate textures
-        self.auxmgr = glimageobject.AuxImageTextureManager(
+        self.auxmgr = textures.AuxImageTextureManager(
             self, clip=None, modulate=None)
         self.auxmgr.registerAuxImage('clip',     self.opts.clipImage)
         self.auxmgr.registerAuxImage('modulate', self.opts.modulateImage)
@@ -382,6 +379,8 @@ class GLVolume(glimageobject.GLImageObject):
         opts    .addListener('invert',           name, self._invertChanged)
         opts    .addListener('modulateAlpha',    name,
                              self._modulateAlphaChanged)
+        opts    .addListener('invertModulateAlpha', name,
+                             self._modulateAlphaChanged)
         opts    .addListener('volume',           name, self._volumeChanged)
         opts    .addListener('channel',          name, self._channelChanged)
         opts    .addListener('interpolation',    name,
@@ -470,6 +469,7 @@ class GLVolume(glimageobject.GLImageObject):
         opts    .removeListener('cmapResolution',          name)
         opts    .removeListener('invert',                  name)
         opts    .removeListener('modulateAlpha',           name)
+        opts    .removeListener('invertModulateAlpha',     name)
         opts    .removeListener('volume',                  name)
         opts    .removeListener('channel',                 name)
         opts    .removeListener('interpolation',           name)
@@ -667,7 +667,7 @@ class GLVolume(glimageobject.GLImageObject):
                                   displayRange=(dmin, dmax))
 
 
-    def preDraw(self, *args, **kwargs):
+    def preDraw(self):
         """Binds the :class:`.ImageTexture` to ``GL_TEXTURE0`` and the
         :class:`.ColourMapTexture` to ``GL_TEXTURE1, and calls the
         version-dependent ``preDraw`` function.
@@ -680,24 +680,24 @@ class GLVolume(glimageobject.GLImageObject):
         self.clipTexture     .bindTexture(gl.GL_TEXTURE3)
         self.modulateTexture .bindTexture(gl.GL_TEXTURE4)
 
-        fslgl.glvolume_funcs.preDraw(self, *args, **kwargs)
+        fslgl.glvolume_funcs.preDraw(self)
 
 
-    def draw2D(self, *args, **kwargs):
+    def draw2D(self, canvas, *args, **kwargs):
         """Calls the version dependent ``draw2D`` function. """
 
         with glroutines.enabled((gl.GL_CULL_FACE)):
             gl.glPolygonMode(gl.GL_FRONT_AND_BACK, gl.GL_FILL)
             gl.glCullFace(gl.GL_BACK)
-            gl.glFrontFace(self.frontFace())
-            fslgl.glvolume_funcs.draw2D(self, *args, **kwargs)
+            gl.glFrontFace(self.frontFace(canvas))
+            fslgl.glvolume_funcs.draw2D(self, canvas, *args, **kwargs)
 
 
-    def draw3D(self, *args, **kwargs):
+    def draw3D(self, canvas, xform=None):
         """Calls the version dependent ``draw3D`` function. """
 
         opts = self.opts
-        w, h = self.canvas.GetScaledSize()
+        w, h = canvas.GetScaledSize()
         res  = self.opts.resolution / 100.0
         sw   = int(np.ceil(w * res))
         sh   = int(np.ceil(h * res))
@@ -705,14 +705,11 @@ class GLVolume(glimageobject.GLImageObject):
         # Initialise and resize
         # the offscreen textures
         for rt in [self.renderTexture1, self.renderTexture2]:
-            if rt.shape != (sw, sh):
-                rt.shape = sw, sh
-
+            rt.shape = sw, sh
             with rt.target():
-                gl.glClearColor(0, 0, 0, 0)
-                gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT)
+                glroutines.clear((0, 0, 0, 0))
 
-        if opts.resolution != 100:
+        if self.opts.resolution != 100:
             gl.glViewport(0, 0, sw, sh)
 
         # Do the render. Even though we're
@@ -728,11 +725,14 @@ class GLVolume(glimageobject.GLImageObject):
             gl.glPolygonMode(gl.GL_FRONT_AND_BACK, gl.GL_FILL)
             gl.glFrontFace(gl.GL_CCW)
             gl.glCullFace(gl.GL_BACK)
-            fslgl.glvolume_funcs.draw3D(self, *args, **kwargs)
+            fslgl.glvolume_funcs.draw3D(self, canvas, xform)
+
+        if self.opts.resolution != 100:
+            gl.glViewport(0, 0, w, h)
 
         # Apply smoothing if needed. If smoothing
-        # is enabled, the final final render will
-        # be in renderTexture2
+        # is enabled, the final render will be in
+        # renderTexture2
         if opts.smoothing > 0:
             self.smoothFilter.set(offsets=[1.0 / sw, 1.0 / sh])
             self.smoothFilter.osApply(self.renderTexture1,
@@ -747,19 +747,13 @@ class GLVolume(glimageobject.GLImageObject):
                           [-1,  1, 0],
                           [ 1,  1, 0]], dtype=np.float32)
 
-        invproj = affine.invert(self.canvas.projectionMatrix)
-        verts   = affine.transform(verts, invproj)
-
-        if opts.resolution != 100:
-            gl.glViewport(0, 0, w, h)
-
         with glroutines.enabled(gl.GL_DEPTH_TEST):
 
             # If smoothing was not applied, rt1
             # contains the final render. Otherwise,
             # rt2 contains the final render, but rt1
             # contains the depth information. So we
-            # need to # temporarily replace rt2.depth
+            # need to temporarily replace rt2.depth
             # with rt1.depth.
             if opts.smoothing > 0:
                 src    = self.renderTexture2
@@ -777,11 +771,10 @@ class GLVolume(glimageobject.GLImageObject):
 
     def drawAll(self, *args, **kwargs):
         """Calls the version dependent ``drawAll`` function. """
-
         fslgl.glvolume_funcs.drawAll(self, *args, **kwargs)
 
 
-    def postDraw(self, *args, **kwargs):
+    def postDraw(self):
         """Unbinds the ``ImageTexture`` and ``ColourMapTexture``, and calls the
         version-dependent ``postDraw`` function.
         """
@@ -792,7 +785,7 @@ class GLVolume(glimageobject.GLImageObject):
         self.clipTexture     .unbindTexture()
         self.modulateTexture .unbindTexture()
 
-        fslgl.glvolume_funcs.postDraw(self, *args, **kwargs)
+        fslgl.glvolume_funcs.postDraw(self)
 
 
     def getAuxTextureXform(self, which):
@@ -828,14 +821,9 @@ class GLVolume(glimageobject.GLImageObject):
         else:
             modXform = self.modulateTexture.voxValXform
 
-        modlo, modhi = opts.modulateRange
-        modrange     = modhi - modlo
-        if modrange == 0:
-            modXform = np.eye(4)
-        else:
-            modXform = affine.concat(
-                affine.scaleOffsetXform(1 / modrange, -modlo / modrange),
-                modXform)
+        modXform = affine.concat(
+            opts.modulateScaleOffset(xform=True),
+            modXform)
 
         return modXform
 
@@ -959,7 +947,8 @@ class GLVolume(glimageobject.GLImageObject):
 
 
     def _modulateAlphaChanged(self, *a):
-        """Called when the :attr:`.VolumeOpts.modulateAlpha` property changes.
+        """Called when the :attr:`.VolumeOpts.modulateAlpha` or
+        :attr:`.VolumeOpts.invertModulateAlpha` properties change.
         Calls :meth:`updateShaderState`.
         """
         self.updateShaderState()
