@@ -10,53 +10,49 @@ sub-classes intended to be used as targets for off-screen rendering.
 
 These classes are used by the :class:`.SliceCanvas` and
 :class:`.LightBoxCanvas` classes for off-screen rendering, and in various
-other situations throughout FSLeyes. See also the
-:class:`.RenderTextureStack`, which uses :class:`RenderTexture` instances.
+other situations throughout FSLeyes.
 """
 
 
 import logging
+import contextlib
 
+import numpy                             as np
 import OpenGL.GL                         as gl
 import OpenGL.raw.GL._types              as gltypes
-import OpenGL.GL.EXT.framebuffer_object  as glfbo
 
-from  fsl.utils.platform import platform as fslplatform
-import fsleyes.gl.routines               as glroutines
-import fsleyes.gl.shaders                as shaders
-from . import                               texture
+import fsl.transform.affine   as affine
+import fsleyes.gl             as fslgl
+import fsleyes.gl.extensions  as glexts
+import fsleyes.gl.routines    as glroutines
+import fsleyes.gl.shaders     as shaders
+from . import                    texture2d
 
 
 log = logging.getLogger(__name__)
 
 
-class RenderTexture(texture.Texture2D):
-    """The ``RenderTexture`` class is a 2D RGBA texture which manages
-    a frame buffer and a render buffer or another ``Texture2D`` which
-    is used as the depth+stencil, or depth attachment.
+class RenderTexture(texture2d.Texture2D):
+    """The ``RenderTexture`` class is a 2D RGBA texture which manages a frame
+    buffer, and a render buffer or a :class:`.DepthTexture` which is used as
+    the depth attachment.
 
     A ``RenderTexture`` is intended to be used as a target for off-screen
     rendering. Using a ``RenderTexture`` (``tex`` in the example below) as the
     rendering target is easy::
 
         # Set the texture size in pixels
-        tex.setSize(1024, 768)
+        tex.shape = 1024, 768
 
         # Bind the texture/frame buffer, and configure
         # the viewport for orthoghraphic display.
         lo = (0.0, 0.0, 0.0)
         hi = (1.0, 1.0, 1.0)
-        tex.bindAsRenderTarget()
-        tex.setRenderViewport(0, 1, lo, hi)
 
-        # ...
-        # draw the scene
-        # ...
-
-        # Unbind the texture/frame buffer,
-        # and restore the previous viewport.
-        tex.unbindAsRenderTarget()
-        tex.restoreViewport()
+        with tex.target(0, 1, lo, hi):
+            # ...
+            # draw the scene
+            # ...
 
 
     The contents of the ``RenderTexture`` can later be drawn to the screen
@@ -66,11 +62,11 @@ class RenderTexture(texture.Texture2D):
 
     A ``RenderTexture`` can be configured in one of three ways:
 
-      1. Using a ``RGBA8`` :class:`Texture2d` (the ``RenderTexture`` itself)
+      1. Using a ``RGBA8`` :class:`.Texture2D` (the ``RenderTexture`` itself)
          as the colour attachment, and no depth or stencil attachments.
 
-      2. As above for the colour attachment, and a ``DEPTH_COMPONENT24``
-         ``Texture2D`` as the depth attachment.
+      2. As above for the colour attachment, and a :class:`.DepthTexture` as
+         the depth attachment.
 
       3. As above for the colour attachment, and a ``DEPTH24_STENCIL8``
          renderbuffer as the combined depth+stencil attachment.
@@ -80,20 +76,21 @@ class RenderTexture(texture.Texture2D):
     implementations. You can choose which option you wish to use via the
     ``rttype`` argument to ``__init__``.
 
-    If you choose option #2, a second ``Texture2D`` instance will be used
-    as the depth attachment. The resulting depth values created after
-    drawing to the ``RenderTexture`` may be used in subsequent processing.
-    You can either access the depth texture directly via the
-    :meth:`getDepthTexture` method, or you can draw the contents of the
-    ``RenderTexture``, with depth information, by passing the ``useDepth``
-    flag to either the :meth:`draw` or :meth:`drawOnBounds` methods.
+    If you choose option #2, a :class:`.DepthTexture` instance will be used as
+    the depth attachment. The resulting depth values created after drawing to
+    the ``RenderTexture`` may be used in subsequent processing.  You can
+    either access the depth texture directly via the :meth:`depthTexture`
+    method, or you can draw the contents of the ``RenderTexture``, with depth
+    information, by passing the ``useDepth`` flag to either the :meth:`draw`
+    or :meth:`drawOnBounds` methods.
     """
 
 
-    def __init__(self, *args, **kwargs):
-        """Create a ``RenderTexture``. All argumenst are passed through
+    def __init__(self, name, rttype='cds', **kwargs):
+        """Create a ``RenderTexture``. All keyword arguments are passed through
         to the :meth:`.Texture2D.__init__` method.
 
+        :arg name:   Unique name for this texture
 
         :arg rttype: ``RenderTexture`` configuration. If provided, must be
                      passed as a keyword argument. Valid values are:
@@ -110,41 +107,44 @@ class RenderTexture(texture.Texture2D):
                   ``RenderTexture``.
         """
 
-        rttype = kwargs.pop('rttype', 'cds')
-
         if rttype not in ('c', 'cd', 'cds'):
             raise ValueError('Invalid rttype: {}'.format(rttype))
 
-        texture.Texture2D.__init__(self, *args, **kwargs)
+        texture2d.Texture2D.__init__(self,
+                                     name,
+                                     ndim=2,
+                                     nvals=4,
+                                     dtype=np.uint8,
+                                     **kwargs)
 
-        self.__frameBuffer     = glfbo.glGenFramebuffersEXT(1)
-        self.__rttype          = rttype
-        self.__renderBuffer    = None
-        self.__depthTexture    = None
-        self.__oldSize         = None
-        self.__oldProjMat      = None
-        self.__oldMVMat        = None
-        self.__oldFrameBuffer  = None
-        self.__oldRenderBuffer = None
+        self.__frameBuffer      = glexts.glGenFramebuffers(1)
+        self.__rttype           = rttype
+        self.__renderBuffer     = None
+        self.__depthTexture     = None
+        self.__oldFrameBuffer   = None
+        self.__oldRenderBuffer  = None
+        self.__oldSize          = None
+        self.__projectionMatrix = None
+        self.__viewMatrix       = None
+        self.__viewport         = None
 
         # Use a single renderbuffer as
         # the depth+stencil attachment
         if rttype == 'cds':
-            self.__renderBuffer = glfbo.glGenRenderbuffersEXT(1)
+            self.__renderBuffer = glexts.glGenRenderbuffers(1)
 
         # Or use a texture as the depth
         # attachment (with no stencil attachment)
         elif rttype == 'cd':
-            self.__depthTexture = texture.Texture2D(
-                '{}_depth'.format(self.getTextureName()),
-                dtype=gl.GL_DEPTH_COMPONENT24)
+            self.__depthTexture = texture2d.DepthTexture(
+                '{}_depth'.format(self.name))
 
             # We also need a shader program in
             # case the creator is intending to
             # use the depth information in draws.
             self.__initShader()
 
-        log.debug('Created fbo {} [{}]'.format(self.__frameBuffer, rttype))
+        log.debug('Created fbo %s [%s]', self.__frameBuffer, rttype)
 
 
     def __initShader(self):
@@ -162,14 +162,14 @@ class RenderTexture(texture.Texture2D):
 
         vertSrc   = shaders.getVertexShader(  'rendertexture')
         fragSrc   = shaders.getFragmentShader('rendertexture')
-        shaderDir = shaders.getShaderDir()
-
-        if float(fslplatform.glVersion) < 2.1:
-            self.__shader = shaders.ARBPShader(vertSrc, fragSrc, shaderDir)
 
+        if float(fslgl.GL_COMPATIBILITY) < 2.1:
+            self.__shader = shaders.ARBPShader(vertSrc,
+                                               fragSrc,
+                                               {'colourTexture' : 0,
+                                                'depthTexture'  : 1})
         else:
             self.__shader = shaders.GLSLShader(vertSrc, fragSrc)
-
             self.__shader.load()
             self.__shader.set('colourTexture', 0)
             self.__shader.set('depthTexture',  1)
@@ -182,16 +182,15 @@ class RenderTexture(texture.Texture2D):
         :meth:`.Texture2D.destroy`.
         """
 
-        texture.Texture2D.destroy(self)
+        texture2d.Texture2D.destroy(self)
 
-        log.debug('Deleting FBO{} [{}]'.format(
-            self.__frameBuffer, self.__rttype))
+        log.debug('Deleting FBO%s [%s]', self.__frameBuffer, self.__rttype)
 
-        glfbo.glDeleteFramebuffersEXT(gltypes.GLuint(self.__frameBuffer))
+        glexts.glDeleteFramebuffers(1, gltypes.GLuint(self.__frameBuffer))
 
         if self.__renderBuffer is not None:
             rb = gltypes.GLuint(self.__renderBuffer)
-            glfbo.glDeleteRenderbuffersEXT(1, rb)
+            glexts.glDeleteRenderbuffers(1, rb)
 
         if self.__depthTexture is not None:
             self.__depthTexture.destroy()
@@ -203,7 +202,42 @@ class RenderTexture(texture.Texture2D):
         self.__oldRenderBuffer = None
 
 
-    def setData(self, data):
+    @property
+    def projectionMatrix(self):
+        """Return the projection matrix to use when drawing to this
+        ``RenderTexture``. Only returns a value when this ``RenderTexture``
+        is bound - returns ``None`` at all other times.
+        """
+        return self.__projectionMatrix
+
+
+    @property
+    def viewMatrix(self):
+        """Return the model-view matrix to use when drawing to this
+        ``RenderTexture``. Only returns a value when this ``RenderTexture``
+        is bound - returns ``None`` at all other times.
+        """
+        return self.__viewMatrix
+
+
+    @property
+    def mvpMatrix(self):
+        """Returns the current model*view*projection matrix. """
+        return affine.concat(self.__projectionMatrix, self.__viewMatrix)
+
+
+    @property
+    def viewport(self):
+        """Return the display coordinate system bounding box for this
+        ``RenderTexture`` as a sequence of three ``(low, high)`` tuples.
+        Only returns a value when this ``RenderTexture`` is bound - returns
+        ``None`` at all other times.
+        """
+        return self.__viewport
+
+
+    @texture2d.Texture2D.data.setter
+    def data(self, data):
         """Raises a :exc:`NotImplementedError`. The ``RenderTexture`` derives
         from the :class:`.Texture2D` class, but is not intended to have its
         texture data manually set - see the :class:`.Texture2D` documentation.
@@ -212,27 +246,67 @@ class RenderTexture(texture.Texture2D):
                                   'instances'.format(type(self).__name__))
 
 
-    def getDepthTexture(self):
-        """Returns the ``Texture2D`` instance used as the depth buffer. Returns
-        ``None`` if this ``RenderTexture`` was not configured with ``'cd'``
-        (see :meth:`__init__`).
+    @property
+    def depthTexture(self):
+        """Returns the ``DepthTexture`` instance used as the depth buffer.
+        Returns ``None`` if this ``RenderTexture`` was not configured with
+        ``'cd'`` (see :meth:`__init__`).
         """
         return self.__depthTexture
 
 
-    def setSize(self, width, height):
-        """Overrides :meth:`.Texture2D.setSize`. Calls that method, and
+    @depthTexture.setter
+    def depthTexture(self, dtex):
+        """Replace the depth texture currently used by this ``RenderTexture``.
+
+        A :exc:`ValueError` is raised if ``dtex`` is not compatible with this
+        ``RenderTexture``.
+
+        :arg dtex: A :class:`.DepthTexture` instance
+        """
+
+        if self.__depthTexture is None:
+            raise ValueError('This RenderTexture is not '
+                             'configured to use a depth texture')
+
+        if not isinstance(dtex, texture2d.DepthTexture)    or \
+           dtex.internalFormat != gl.GL_DEPTH_COMPONENT24  or \
+           dtex.shape          != self.shape:
+            raise ValueError('Incompatible depth texture')
+
+        self.__depthTexture = dtex
+
+
+    @texture2d.Texture2D.shape.setter
+    def shape(self, shape):
+        """Overrides the :meth:`.Texture2D.shape` setter. Calls that method, and
         also calls it on the depth texture if it exists.
         """
 
-        # We have to size the depth texture first,
-        # because calling setSize on ourselves will
-        # result in refresh being called, whcih
-        # expects the depth texture to be ready to go.
-        if self.__depthTexture is not None:
-            self.__depthTexture.setSize(width, height)
+        width, height = shape
 
-        texture.Texture2D.setSize(self, width, height)
+        # We have to size the depth texture first,
+        # because calling shape on ourselves will
+        # result in refresh being called, which
+        # expects the depth texture to be ready to
+        # go.
+        if self.__depthTexture is not None:
+            self.__depthTexture.shape = width, height
+
+        texture2d.Texture2D.shape.fset(self, (width, height))
+
+
+    @contextlib.contextmanager
+    def renderViewport(self, xax, yax, lo, hi):
+        """Context manager which sets and restores the viewport via
+        :meth:`setRenderViewport` and :meth:`restoreViewport`.
+        """
+
+        self.setRenderViewport(xax, yax, lo, hi)
+        try:
+            yield
+        finally:
+            self.restoreViewport()
 
 
     def setRenderViewport(self, xax, yax, lo, hi):
@@ -255,25 +329,25 @@ class RenderTexture(texture.Texture2D):
                   coordinates.
         """
 
-        if self.__oldSize    is not None or \
-           self.__oldProjMat is not None or \
-           self.__oldMVMat   is not None:
+        if self.__oldSize is not None:
             raise RuntimeError('RenderTexture RB{}/FBO{} has already '
                                'configured the viewport'.format(
                                    self.__renderBuffer,
                                    self.__frameBuffer))
 
-        log.debug('Configuring viewport for RB{}/FBO{}'.format(
-            self.__renderBuffer,
-            self.__frameBuffer))
+        log.debug('Configuring viewport for RB%s/FBO%s',
+                  self.__renderBuffer, self.__frameBuffer)
 
-        width, height = self.getSize()
+        self.__oldSize = gl.glGetIntegerv(gl.GL_VIEWPORT)
 
-        self.__oldSize    = gl.glGetIntegerv(gl.GL_VIEWPORT)
-        self.__oldProjMat = gl.glGetFloatv(  gl.GL_PROJECTION_MATRIX)
-        self.__oldMVMat   = gl.glGetFloatv(  gl.GL_MODELVIEW_MATRIX)
+        width, height  = self.shape
+        projmat, mvmat = glroutines.show2D(xax, yax, lo, hi)
 
-        glroutines.show2D(xax, yax, width, height, lo, hi)
+        gl.glViewport(0, 0, width, height)
+
+        self.__viewport         = list(zip(lo, hi))
+        self.__projectionMatrix = projmat
+        self.__viewMatrix       = mvmat
 
 
     def restoreViewport(self):
@@ -281,27 +355,47 @@ class RenderTexture(texture.Texture2D):
         to :meth:`setRenderViewport`.
         """
 
-        if self.__oldSize    is None or \
-           self.__oldProjMat is None or \
-           self.__oldMVMat   is None:
+        if self.__oldSize is None:
             raise RuntimeError('RenderTexture RB{}/FBO{} has not '
                                'configured the viewport'.format(
                                    self.__renderBuffer,
                                    self.__frameBuffer))
 
-        log.debug('Clearing viewport (from RB{}/FBO{})'.format(
-            self.__renderBuffer,
-            self.__frameBuffer))
+        log.debug('Clearing viewport (from RB%s/FBO%s)',
+                  self.__renderBuffer, self.__frameBuffer)
 
         gl.glViewport(*self.__oldSize)
-        gl.glMatrixMode(gl.GL_PROJECTION)
-        gl.glLoadMatrixf(self.__oldProjMat)
-        gl.glMatrixMode(gl.GL_MODELVIEW)
-        gl.glLoadMatrixf(self.__oldMVMat)
+        self.__oldSize          = None
+        self.__projectionMatrix = None
+        self.__viewMatrix       = None
+        self.__viewport         = None
 
-        self.__oldSize    = None
-        self.__oldProjMat = None
-        self.__oldMVMat   = None
+
+    @contextlib.contextmanager
+    def target(self, *args, **kwargs):
+        """Context manager which binds and unbinds this ``RenderTexture`` as
+        the render target, via :meth:`bindAsRenderTarget` and
+        :meth:`unbindAsRenderTarget`.
+
+        If any arguments are provided, the viewport is also set and restored
+        via :meth:`setRenderViewport` and :meth:`restoreViewport`.
+        """
+
+        setViewport  = len(args) > 0 or len(kwargs) > 0
+        alreadyBound = self.__oldFrameBuffer is not None
+
+        if not alreadyBound:
+            self.bindAsRenderTarget()
+
+        if setViewport:
+            self.setRenderViewport(*args, **kwargs)
+        try:
+            yield
+        finally:
+            if setViewport:
+                self.restoreViewport()
+            if not alreadyBound:
+                self.unbindAsRenderTarget()
 
 
     def bindAsRenderTarget(self):
@@ -317,20 +411,20 @@ class RenderTexture(texture.Texture2D):
                                'bound'.format(self.__frameBuffer))
 
         self.__oldFrameBuffer  = gl.glGetIntegerv(
-            glfbo.GL_FRAMEBUFFER_BINDING_EXT)
+            glexts.GL_FRAMEBUFFER_BINDING)
 
         if self.__renderBuffer is not None:
             self.__oldRenderBuffer = gl.glGetIntegerv(
-                glfbo.GL_RENDERBUFFER_BINDING_EXT)
+                glexts.GL_RENDERBUFFER_BINDING)
 
-        log.debug('Setting FBO{} as render target'.format(self.__frameBuffer))
+        log.debug('Setting FBO%s as render target', self.__frameBuffer)
 
-        glfbo.glBindFramebufferEXT(
-            glfbo.GL_FRAMEBUFFER_EXT, self.__frameBuffer)
+        glexts.glBindFramebuffer(
+            glexts.GL_FRAMEBUFFER, self.__frameBuffer)
 
         if self.__renderBuffer is not None:
-            glfbo.glBindRenderbufferEXT(
-                glfbo.GL_RENDERBUFFER_EXT, self.__renderBuffer)
+            glexts.glBindRenderbuffer(
+                glexts.GL_RENDERBUFFER, self.__renderBuffer)
 
 
     def unbindAsRenderTarget(self):
@@ -342,90 +436,87 @@ class RenderTexture(texture.Texture2D):
             raise RuntimeError('RenderTexture FBO{} has not been '
                                'bound'.format(self.__frameBuffer))
 
-        log.debug('Restoring render target to FBO{} '
-                  '(from FBO{})'.format(
-                      self.__oldFrameBuffer,
-                      self.__frameBuffer))
+        log.debug('Restoring render target to FBO%s (from FBO%s)',
+                  self.__oldFrameBuffer, self.__frameBuffer)
 
-        glfbo.glBindFramebufferEXT(
-            glfbo.GL_FRAMEBUFFER_EXT, self.__oldFrameBuffer)
+        glexts.glBindFramebuffer(
+            glexts.GL_FRAMEBUFFER, self.__oldFrameBuffer)
 
         if self.__renderBuffer is not None:
-            glfbo.glBindRenderbufferEXT(
-                glfbo.GL_RENDERBUFFER_EXT, self.__oldRenderBuffer)
+            glexts.glBindRenderbuffer(
+                glexts.GL_RENDERBUFFER, self.__oldRenderBuffer)
 
         self.__oldFrameBuffer  = None
         self.__oldRenderBuffer = None
 
 
-    def refresh(self):
-        """Overrides :meth:`.Texture2D.refresh`. Calls the base-class
+    def doRefresh(self):
+        """Overrides :meth:`.Texture2D.doRefresh`. Calls the base-class
         implementation, and ensures that the frame buffer and render buffer
         of this ``RenderTexture`` are configured correctly.
         """
-        texture.Texture2D.refresh(self)
+        texture2d.Texture2D.doRefresh(self)
 
-        width, height = self.getSize()
+        width, height = self.shape
+
+        log.debug('Refreshing render texture FBO%s', self.__frameBuffer)
 
         # Bind the colour buffer
-        self.bindAsRenderTarget()
-        glfbo.glFramebufferTexture2DEXT(
-            glfbo.GL_FRAMEBUFFER_EXT,
-            glfbo.GL_COLOR_ATTACHMENT0_EXT,
-            gl   .GL_TEXTURE_2D,
-            self.getTextureHandle(),
-            0)
-
-        # Combined depth/stencil attachment
-        if self.__rttype == 'cds':
-
-            # Configure the render buffer
-            glfbo.glRenderbufferStorageEXT(
-                glfbo.GL_RENDERBUFFER_EXT,
-                gl.GL_DEPTH24_STENCIL8,
-                width,
-                height)
-
-            # Bind the render buffer
-            glfbo.glFramebufferRenderbufferEXT(
-                glfbo.GL_FRAMEBUFFER_EXT,
-                gl.GL_DEPTH_STENCIL_ATTACHMENT,
-                glfbo.GL_RENDERBUFFER_EXT,
-                self.__renderBuffer)
-
-        # Or a depth texture
-        elif self.__rttype == 'cd':
-
-            glfbo.glFramebufferTexture2DEXT(
-                glfbo.GL_FRAMEBUFFER_EXT,
-                glfbo.GL_DEPTH_ATTACHMENT_EXT,
+        with self.target():
+            glexts.glFramebufferTexture2D(
+                glexts.GL_FRAMEBUFFER,
+                glexts.GL_COLOR_ATTACHMENT0,
                 gl   .GL_TEXTURE_2D,
-                self.__depthTexture.getTextureHandle(),
+                self.handle,
                 0)
 
-        # Get the FBO status before unbinding it -
-        # the Apple software renderer will return
-        # FRAMEBUFFER_UNDEFINED otherwise.
-        status = glfbo.glCheckFramebufferStatusEXT(glfbo.GL_FRAMEBUFFER_EXT)
+            # Combined depth/stencil attachment
+            if self.__rttype == 'cds':
 
-        self.unbindAsRenderTarget()
+                # Configure the render buffer
+                glexts.glRenderbufferStorage(
+                    glexts.GL_RENDERBUFFER,
+                    gl.GL_DEPTH24_STENCIL8,
+                    width,
+                    height)
+
+                # Bind the render buffer
+                glexts.glFramebufferRenderbuffer(
+                    glexts.GL_FRAMEBUFFER,
+                    gl.GL_DEPTH_STENCIL_ATTACHMENT,
+                    glexts.GL_RENDERBUFFER,
+                    self.__renderBuffer)
+
+            # Or a depth texture
+            elif self.__rttype == 'cd':
+                glexts.glFramebufferTexture2D(
+                    glexts.GL_FRAMEBUFFER,
+                    glexts.GL_DEPTH_ATTACHMENT,
+                    gl   .GL_TEXTURE_2D,
+                    self.__depthTexture.handle,
+                    0)
+
+            # Get the FBO status before unbinding it -
+            # the Apple software renderer will return
+            # FRAMEBUFFER_UNDEFINED otherwise.
+            status = glexts.glCheckFramebufferStatus(
+                glexts.GL_FRAMEBUFFER)
 
         # Complain if something is not right
-        if status != glfbo.GL_FRAMEBUFFER_COMPLETE_EXT:
+        if status != glexts.GL_FRAMEBUFFER_COMPLETE:
             raise RuntimeError('An error has occurred while configuring '
                                'the frame buffer [{}]'.format(status))
 
 
-    def draw(self, *args, **kwargs):
+    def draw(self, vertices, useDepth=False):
         """Overrides :meth:`.Texture2D.draw`. Calls that method, optionally
         using the information in the depth texture.
 
 
-        :arg useDepth: Must be passed as a keyword argument. Defaults to
-                       ``False``. If ``True``, and this ``RenderTexture``
-                       was configured to use a depth texture, the texture
-                       is rendered with depth information using a fragment
-                       program
+        :arg useDepth: Defaults to ``False``. If ``True``, and this
+                       ``RenderTexture`` was configured to use a depth
+                       texture, the texture is rendered with depth
+                       information using a fragment program.
 
 
         A ``RuntimeError`` will be raised if ``useDepth is True``, but this
@@ -433,21 +524,24 @@ class RenderTexture(texture.Texture2D):
         setting in :meth:`__init__`).
         """
 
-        useDepth = kwargs.pop('useDepth', False)
-
         if useDepth and self.__depthTexture is None:
             raise RuntimeError('useDepth is True but I don\'t '
                                'have a depth texture!')
 
-        if useDepth:
-            self.__depthTexture.bindTexture(gl.GL_TEXTURE1)
-            self.__shader.load()
-
-        texture.Texture2D.draw(self, *args, **kwargs)
-
-        if useDepth:
-            self.__shader.unload()
-            self.__depthTexture.unbindTexture()
+        # Draw using the default Texture2D shader
+        if not useDepth:
+            texture2d.Texture2D.draw(self, vertices)
+        else:
+            shader    = self.__shader
+            depthTex  = self.__depthTexture
+            texCoords = self.generateTextureCoords()
+            nverts    = vertices.shape[0]
+            with shader.loaded():
+                shader.setAtt('vertex',   vertices)
+                shader.setAtt('texCoord', texCoords)
+                with self    .bound(gl.GL_TEXTURE0), \
+                     depthTex.bound(gl.GL_TEXTURE1):
+                    shader.draw(gl.GL_TRIANGLES, 0, nverts)
 
 
 class GLObjectRenderTexture(RenderTexture):
@@ -492,10 +586,10 @@ class GLObjectRenderTexture(RenderTexture):
 
         RenderTexture.__init__(self, name)
 
-        name = '{}_{}'.format(self.getTextureName(), id(self))
-        globj.register(name, self.__updateSize)
+        name = '{}_{}'.format(self.name, id(self))
+        globj.register(name, self.__updateShape)
 
-        self.__updateSize()
+        self.__updateShape()
 
 
     def destroy(self):
@@ -504,7 +598,7 @@ class GLObjectRenderTexture(RenderTexture):
         calls :meth:`.RenderTexture.destroy`.
         """
 
-        name = '{}_{}'.format(self.getTextureName(), id(self))
+        name = '{}_{}'.format(self.name, id(self))
         self.__globj.deregister(name)
         RenderTexture.destroy(self)
 
@@ -517,19 +611,21 @@ class GLObjectRenderTexture(RenderTexture):
         """
         self.__xax = xax
         self.__yax = yax
-        self.__updateSize()
+        self.__updateShape()
 
 
-    def setSize(self, width, height):
-        """Raises a :exc:`NotImplementedError`. The size of a
-        ``GLObjectRenderTexture`` is set automatically.
+    @RenderTexture.shape.setter
+    def shape(self, shape):
+        """Overrides the :meth:`.Texture.shape` setter. Raises a
+        :exc:`NotImplementedError`. The size of a ``GLObjectRenderTexture`` is
+        set automatically.
         """
         raise NotImplementedError(
             'Texture size cannot be set for {} instances'.format(
                 type(self).__name__))
 
 
-    def __updateSize(self, *a):
+    def __updateShape(self, *a):
         """Updates the size of this ``GLObjectRenderTexture``, basing it
         on the resolution returned by the :meth:`.GLObject.getDataResolution`
         method. If that method returns ``None``, a default resolution is used.
@@ -549,10 +645,8 @@ class GLObjectRenderTexture(RenderTexture):
             resolution[self.__xax] = width
             resolution[self.__yax] = height
 
-            log.debug('Using default resolution '
-                      'for GLObject {}: {}'.format(
-                          type(globj).__name__,
-                          resolution))
+            log.debug('Using default resolution for GLObject %s: %s',
+                      type(globj).__name__, resolution)
 
         width  = resolution[self.__xax]
         height = resolution[self.__yax]
@@ -574,7 +668,7 @@ class GLObjectRenderTexture(RenderTexture):
             width  = int(round(width))
             height = int(round(height))
 
-        log.debug('Setting {} texture resolution to {}x{}'.format(
-            type(globj).__name__, width, height))
+        log.debug('Setting %s texture resolution to %sx%s',
+                  type(globj).__name__, width, height)
 
-        RenderTexture.setSize(self, width, height)
+        RenderTexture.shape.fset(self, (width, height))
