@@ -29,7 +29,7 @@ from . import data                  as texdata
 log = logging.getLogger(__name__)
 
 
-class TextureBase(object):
+class TextureBase:
     """Base mixin class used by the :class:`Texture` class.
 
     This class provides logic for texture lifecycle management
@@ -209,7 +209,7 @@ class TextureBase(object):
         self.__bound = max(0, self.__bound - 1)
 
 
-class TextureSettingsMixin(object):
+class TextureSettingsMixin:
     """Mixin class used by the :class:`Texture` class.
 
 
@@ -505,6 +505,7 @@ class Texture(notifier.Notifier, TextureBase, TextureSettingsMixin):
                  settings=None,
                  textureFormat=None,
                  internalFormat=None,
+                 initialise=True,
                  **kwargs):
         """Create a ``Texture``.
 
@@ -531,8 +532,13 @@ class Texture(notifier.Notifier, TextureBase, TextureSettingsMixin):
         :arg internalFormat: Internal texture format to use - if not specified,
                              this is automatically determined.
 
+        :arg initialise:     If ``True`` (the default), an initial call to
+                             :meth:`set` is made. Setting this to ``False``
+                             allows for two-stage initialisation, if needed
+                             (e.g. to obtain a reference to the ``Texture``).
+
         All other arguments are passed through to the initial call to
-        :meth:`set`.
+        :meth:`set` (unless ``initialise is False``).
 
         .. note:: All subclasses must accept a ``name`` as the first parameter
                   to their ``__init__`` method, and must pass said ``name``
@@ -596,7 +602,8 @@ class Texture(notifier.Notifier, TextureBase, TextureSettingsMixin):
             self.__taskThread = None
             self.__taskName   = None
 
-        self.set(**kwargs)
+        if initialise:
+            self.set(**kwargs)
 
 
     def destroy(self):
@@ -751,7 +758,8 @@ class Texture(notifier.Notifier, TextureBase, TextureSettingsMixin):
         ``callback``       Optional function which will be called (via
                            :func:`.idle.idle`) when the texture has been
                            refreshed. Only called if ``refresh`` is
-                           ``True``, and a setting has changed.
+                           ``True``, and a setting has changed. Passed
+                           a reference to this Texture object.
         ``notify``         Passed through to the :meth:`refresh` method.
         ================== ==============================================
 
@@ -847,7 +855,8 @@ class Texture(notifier.Notifier, TextureBase, TextureSettingsMixin):
         :arg callback:     Optional function which will be called (via
                            :func:`.idle.idle`) when the texture has been
                            refreshed. Only called if ``refresh`` is
-                           ``True``, and a setting has changed.
+                           ``True``, and a setting has changed. Passed
+                           a reference to this Texture object.
 
         .. note:: The texture data may be generated on a separate thread, using
                   the :func:`.idle.run` function. This is controlled by the
@@ -900,7 +909,7 @@ class Texture(notifier.Notifier, TextureBase, TextureSettingsMixin):
             if notify:
                 self.notify()
             if callback is not None:
-                callback()
+                callback(self)
 
         # Wrap the above functions in a report
         # decorator in case an error occurs
