@@ -189,6 +189,14 @@ class FSLeyesApp(wx.App):
         # queue the files to open them later
         # in SetOverlayListAndDisplayContext
         if self.__overlayList is None:
+
+            # On certain systems (observed on Big Sur), when
+            # files are passed as command-line arguments, they
+            # are passed to MacOpenFiles. We don't want that,
+            # because cli arguments are parsed separately. So
+            # we remove dupes here.
+
+            filenames = [f for f in filenames if f not in sys.argv]
             self.__filesToOpen.extend(filenames)
             return
 
@@ -382,7 +390,7 @@ def main(args=None):
         # get swamped with update notifications.
         if namespace[0].updatecheck:
             import fsleyes.actions.updatecheck as updatecheck
-            wx.CallAfter(updatecheck.UpdateCheckAction(),
+            wx.CallAfter(updatecheck.UpdateCheckAction(overlayList, displayCtx),
                          showUpToDateMessage=False,
                          showErrorMessage=False,
                          ignorePoint=False)
@@ -576,7 +584,8 @@ def initialise(splash, namespace, callback):
     try:
         # Force the creation of a wx.glcanvas.GLContext object,
         # and initialise OpenGL version-specific module loads.
-        fslgl.getGLContext(ready=realCallback)
+        fslgl.getGLContext(ready=realCallback,
+                           requestVersion=namespace.glversion)
 
     except Exception:
         log.error('Unable to initialise OpenGL!', exc_info=True)
@@ -818,13 +827,6 @@ def makeFrame(namespace, displayCtx, overlayList, splash, closeHandlers):
 
     status.update('Setting up scene...')
 
-    # Set the default SceneOpts.performance
-    # level so that all created SceneOpts
-    # instances will default to it
-    if namespace.performance is not None:
-        fsldisplay.SceneOpts.performance.setAttribute(
-            None, 'default', namespace.performance)
-
     # If a layout has been specified,
     # we load the layout
     if namespace.scene is not None:
