@@ -39,9 +39,9 @@ an overlay type are:
 One further requirement is imposed on overlay types which derive from the
 :class:`.Image` class:
 
- -  The ``__init__`` method fo ll sub-classes of the ``Image`` class must
-    accept the ``loadData``, ``calcRange``, ``indexed`` , and ``threaded``
-    parameters, and pass them through to the base class ``__init__`` method.
+ -  The ``__init__`` method for sub-classes of the ``Image`` class must
+    accept the ``loadMeta`` and ``dataMgr`` parameters, and pass them
+    through to the base class ``__init__`` method.
 
 
 Currently (``fsleyes`` version |version|) the only overlay types in existence
@@ -58,6 +58,7 @@ Currently (``fsleyes`` version |version|) the only overlay types in existence
    ~fsl.data.vtk.VTKMesh
    ~fsl.data.gifti.GiftiMesh
    ~fsl.data.freesurfer.FreesurferMesh
+   ~fsl.data.bitmap.Bitmap
 
 
 This module also provides a few convenience classes and functions:
@@ -67,7 +68,6 @@ This module also provides a few convenience classes and functions:
    :nosignatures:
 
    ProxyImage
-   guessDataSourceType
    findFEATImage
 """
 
@@ -75,9 +75,9 @@ This module also provides a few convenience classes and functions:
 import os.path as op
 import            logging
 import            weakref
-import            deprecation
+import            collections
 
-import fsl.utils.path as fslpath
+import fsl.data.utils as dutils
 import fsl.data.image as fslimage
 import fsleyes_props  as props
 
@@ -124,12 +124,21 @@ class OverlayList(props.HasProperties):
         if overlays is None: overlays = []
         self.overlays.extend(overlays)
 
-        # The append/insert methods allow an initial
-        # overlay type to be specified for newly
-        # added overlays. This can be queried via
-        # the initOverlayType method (and is done so
-        # by DisplayContext instances).
-        self.__initOverlayType = {}
+        # The append/insert methods allow display/opts
+        # property values to be specified for newly
+        # added overlays. These can be queried via
+        # the initProps method (and is done so by
+        # DisplayContext/Display instances).
+        # It is a dict of dicts:
+        #
+        #    {
+        #      overlay : {
+        #        propName: value,
+        #        propName: value,
+        #      },
+        #      ...
+        #    }
+        self.__initProps = collections.defaultdict(dict)
 
         # This dictionary may be used throughout FSLeyes,
         # via the getData/setData methods, to store
@@ -149,12 +158,16 @@ class OverlayList(props.HasProperties):
         self.__overlayData = weakref.WeakKeyDictionary()
 
 
-    def initOverlayType(self, overlay):
-        """Returns the initial type for the given ``overlay``, if it was
-        specified via the :meth:`append` or :meth:`insert` methods. Returns
-        ``None`` otherwise.
+    def initProps(self, overlay):
+        """Returns a dict containing initial :class:`.Display` and
+        :class:`.DisplayOpts` property values to be used for the given
+        ``overlay``, if they were specified via the :meth:`append` or
+        :meth:`insert` methods.
+
+        This method requires that there is no overlap between the property
+        names used in :class:`.Display` and :class:`.DisplayOpts` classes.
         """
-        return self.__initOverlayType.get(overlay, None)
+        return self.__initProps[overlay]
 
 
     def getData(self, overlay, key, *args):
@@ -249,7 +262,7 @@ class OverlayList(props.HasProperties):
 
         ovls = self[key]
         for ovl in ovls:
-            self.__initOverlayType.pop(ovl, None)
+            self.__initProps.pop(ovl, None)
 
         return self.overlays.__delitem__(key)
 
@@ -259,56 +272,62 @@ class OverlayList(props.HasProperties):
     def count(self, item):
         return self.overlays.count(item)
 
-    def append(self, item, overlayType=None):
+    def insert(self, index, item, **initProps):
+        """Insert a new overlay into the overlay list.
+
+        Any initial :class:`.Display`/:class:`.DisplayOpts` property values
+        may be passed in as keyword arguments.
+        """
 
         with props.suppress(self, 'overlays', notify=True):
+            self.overlays.insert(index, item)
+            self.__initProps[item] = initProps
 
-            self.overlays.append(item)
+    def append(self, item, **initProps):
+        """Add a new overlay to the end of the overlay list.
 
-            if overlayType is not None:
-                self.__initOverlayType[item] = overlayType
+        Any initial :class:`.Display`/:class:`.DisplayOpts` property values
+        may be passed in as keyword arguments.
+        """
+        self.insert(len(self), item, **initProps)
 
+    def extend(self, iterable, **initProps):
+        """Add new overlays to the overlay list.
 
-    def extend(self, iterable, overlayTypes=None):
+        Any initial :class:`.Display`/:class:`.DisplayOpts` property values
+        may be passed in as keyword arguments, where the argument name is the
+        property name, and the argument value is a dict of
+        ``{overlay : value}`` mappings.
+        """
 
         with props.suppress(self, 'overlays', notify=True):
 
             result = self.overlays.extend(iterable)
 
-            if overlayTypes is not None:
-                for overlay, overlayType in overlayTypes.items():
-                    self.__initOverlayType[overlay] = overlayType
+            for propName, overlayProps in initProps.items():
+                for overlay, val in overlayProps.items():
+                    self.__initProps[overlay][propName] = val
 
         return result
 
+    def insertAll(self, index, items):
+        return self.overlays.insertAll(index, items)
+
     def pop(self, index=-1):
         ovl = self.overlays.pop(index)
-        self.__initOverlayType.pop(ovl, None)
+        self.__initProps.pop(ovl, None)
         return ovl
 
     def move(self, from_, to):
         return self.overlays.move(from_, to)
 
     def remove(self, item):
-        self.__initOverlayType.pop(item, None)
         self.overlays.remove(item)
+        self.__initProps.pop(item, None)
 
     def clear(self):
         del self[:]
 
-    def insert(self, index, item, overlayType=None):
-
-        with props.suppress(self, 'overlays', notify=True):
-
-            self.overlays.insert(index, item)
-
-            if overlayType is not None:
-                self.__initOverlayType[item] = overlayType
-
-
-    def insertAll(self, index, items):
-        return self.overlays.insertAll(index, items)
-
 
 class ProxyImage(fslimage.Image):
     """The ``ProxyImage`` class is a simple wrapper around an :class:`Image`
@@ -346,199 +365,6 @@ class ProxyImage(fslimage.Image):
         return self.__base
 
 
-class PropCache(object):
-    """Deprecated - use :class:`fsleyes_props.PropCache` instead.
-
-    A little convenience class which can be used to track and cache
-    property values, related to each overlay in the :class:`.OverlayList`,
-    on some :class:`.HasProperties` object.
-
-
-    Whenever the selected overlay changes, the property values of the
-    previously selected overlay are cached. Later on, when that overlay
-    is re-selected, the cached property values may be retrieved via
-    the :meth:`get` method.
-    """
-
-
-    @deprecation.deprecated(deprecated_in='0.22.0',
-                            removed_in='1.0.0',
-                            details='Use fsleyes_props.PropCache instead')
-    def __init__(self, overlayList, displayCtx, target, propNames):
-        """Create a ``PropCache``.
-
-        :arg overlayList: The :class:`.OverlayList`.
-
-        :arg displayCtx:  The :class:`.DisplayContext` instance.
-
-        :arg target:      The :class:`.HasProperties` instance containing
-                          the properties that are to be cached.
-
-        :arg propNames:   List containing the names of ``target`` properties
-                          to be cached.
-        """
-
-        self.__name           = '{}_{}'.format(type(self).__name__, id(self))
-        self.__overlayList    = overlayList
-        self.__displayCtx     = displayCtx
-        self.__target         = target
-        self.__propNames      = propNames
-        self.__currentOverlay = None
-
-        # { (overlay, propName) : value }
-        self.__cache          = {}
-
-        self.__overlayList.addListener('overlays',
-                                       self.__name,
-                                       self.__selectedOverlayChanged)
-        self.__displayCtx .addListener('selectedOverlay',
-                                       self.__name,
-                                       self.__selectedOverlayChanged)
-
-        self.__selectedOverlayChanged()
-
-
-    def destroy(self):
-        """Must be called when this ``PropCache`` is no longer needed.
-        Removes property listeners and clears references.
-        """
-        self.__overlayList.removeListener('overlays',        self.__name)
-        self.__displayCtx .removeListener('selectedOverlay', self.__name)
-
-        self.__overlayList    = None
-        self.__displayCtx     = None
-        self.__currentOverlay = None
-        self.__cache          = None
-
-
-    def get(self, overlay, propName, *args):
-        """Returns the cached property value for the specified ``overlay``.
-        If there is no cached property value for the overlay, the specified
-        ``default`` value is returned. If a ``default`` value is not
-        provided, the current property value is returned.
-
-        :arg overlay:  Overlay to retrieve the property value for
-
-        :arg propName: Name of the property to return a value for
-
-        :arg default:  Value to return if a value for the the
-                       overlay/property is not cached
-        """
-
-        if len(args) not in (0, 1):
-            raise ValueError('Invalid arguments passed to PropCache.get')
-
-        defaultProvided = len(args) == 1
-
-        val = self.__cache.get((overlay, propName), None)
-
-        if val is None:
-            if defaultProvided: val = args[0]
-            else:               val = getattr(self.__target, propName)
-
-        return val
-
-
-    def __selectedOverlayChanged(self, *a):
-        """Called when either the :attr:`.DisplayContext.selectedOverlay`,
-        or the :attr:`.OverlayList.overlays` change. Caches property values
-        pertaining to the previously selected overlay.
-        """
-
-        self.__cacheGC()
-
-        oldOverlay = self.__currentOverlay
-        newOverlay = self.__displayCtx.getSelectedOverlay()
-
-        self.__currentOverlay = newOverlay
-
-        if oldOverlay     is None       or \
-           oldOverlay     is newOverlay or \
-           oldOverlay not in self.__overlayList:
-            return
-
-        for prop in self.__propNames:
-            self.__cache[oldOverlay, prop] = getattr(self.__target, prop)
-
-
-    def __cacheGC(self):
-        """Removes any obsolete entries (those pertaining to overlays which
-        are no longer present in the :class:`.OverlayList`) from the cache.
-        """
-
-        keys = list(self.__cache.keys())
-
-        for overlay, prop in keys:
-            if overlay not in self.__overlayList:
-                self.__cache.pop((overlay, prop))
-
-
-def guessDataSourceType(path):
-    """A convenience function which, given the name of a file or directory,
-    figures out a suitable overlay type.
-
-    Returns a tuple containing two values - a type which should be able to
-    load the path, and the path itself, possibly adjusted. If the type
-    is unrecognised, the first tuple value will be ``None``.
-    """
-
-    import fsl.data.vtk             as fslvtk
-    import fsl.data.gifti           as fslgifti
-    import fsl.data.freesurfer      as fslfs
-    import fsl.data.mghimage        as fslmgh
-    import fsl.data.featimage       as featimage
-    import fsl.data.melodicimage    as melimage
-    import fsl.data.dtifit          as dtifit
-    import fsl.data.melodicanalysis as melanalysis
-    import fsl.data.featanalysis    as featanalysis
-
-    # Support files opened via fsleyes:// URL
-    if path.startswith('fsleyes://'):
-        path = path[10:]
-
-    path = op.abspath(path)
-
-    if op.isfile(path):
-
-        # Some types are easy - just check the extensions
-        if fslpath.hasExt(path, fslvtk.ALLOWED_EXTENSIONS):
-            return fslvtk.VTKMesh, path
-        elif fslpath.hasExt(path, fslgifti.ALLOWED_EXTENSIONS):
-            return fslgifti.GiftiMesh, path
-        elif fslfs.isGeometryFile(path):
-            return fslfs.FreesurferMesh, path
-        elif fslpath.hasExt(path, fslmgh.ALLOWED_EXTENSIONS):
-            return fslmgh.MGHImage, path
-
-        # Other specialised image types
-        elif melanalysis .isMelodicImage(path):
-            return melimage.MelodicImage, path
-        elif featanalysis.isFEATImage(   path):
-            return featimage.FEATImage,   path
-        elif fslimage.looksLikeImage(path):
-            return fslimage.Image, path
-
-    # Analysis directory?
-    elif op.isdir(path):
-        if melanalysis.isMelodicDir(path):
-            return melimage.MelodicImage, path
-        elif featanalysis.isFEATDir(path):
-            return featimage.FEATImage, path
-        elif dtifit.isDTIFitPath(path):
-            return dtifit.DTIFitTensor, path
-
-    # Last-ditch effort - is it an
-    # incomplete path to an image?
-    try:
-        path = fslimage.addExt(path, mustExist=True)
-        return fslimage.Image, path
-
-    # Otherwise, I don't
-    # know what to do
-    except fslimage.PathError:
-        return None, path
-
-
 def findFEATImage(overlayList, overlay):
     """Searches the given :class:`.OverlayList` to see if there is a
     :class:`.FEATImage` associated with the given ``overlay``. Returns the
@@ -583,7 +409,7 @@ def findMeshReferenceImage(overlayList, overlay):
             if prefix.startswith(ovl.name):
                 return ovl
 
-    except:
+    except Exception:
         pass
 
     return None
