@@ -156,7 +156,6 @@ class FileTreePanel(ctrlpanel.ControlPanel):
         treefiles   = [tf for tf in filetree.list_all_trees() if filter(tf)]
         treefiles   = list(sorted(treefiles))
         treefiles  += FileTreePanel.customTrees
-        treefiles   = [op.abspath( tf) for tf in treefiles]
         treelabels  = [op.basename(tf) for tf in treefiles]
 
         for f, l in zip(treefiles, treelabels):
@@ -440,7 +439,7 @@ class FileTypePanel(elb.EditableListBox):
             toggle = wx.CheckBox(self)
             toggle.SetMinSize(toggle.GetBestSize())
             self.Append(ftype, extraWidget=toggle)
-            toggle.Bind(wx.EVT_CHECKBOX, self.__onToggle)
+            toggle.Bind(wx.EVT_CHECKBOX, self.onToggle)
 
 
     def GetFileTypes(self):
@@ -457,7 +456,7 @@ class FileTypePanel(elb.EditableListBox):
         return active
 
 
-    def __onToggle(self, ev):
+    def onToggle(self, ev):
         """Called when a file type is toggled. Calls the
         :meth:`FileTreePanel.Update` method.
         """
