@@ -60,8 +60,15 @@ def main(args=None, hook=None):
     # Initialise colour maps module
     fslcm.init()
 
+    # Parse arguments, and
+    # configure logging/debugging
+    namespace = parseArgs(args)
+    fsleyes.configLogging(namespace.verbose, namespace.noisy)
+
     # Create a GL context
-    fslgl.getGLContext(offscreen=True, createApp=True)
+    fslgl.getGLContext(offscreen=True,
+                       createApp=True,
+                       requestVersion=namespace.glversion)
 
     # Now that GL inititalisation is over,
     # make sure that the idle loop executes
@@ -73,10 +80,6 @@ def main(args=None, hook=None):
     with idle.idleLoop.synchronous(), \
          imagetexture.ImageTexture.enableThreading(False):
 
-        # Parse arguments, and
-        # configure logging/debugging
-        namespace = parseArgs(args)
-        fsleyes.configLogging(namespace.verbose, namespace.noisy)
 
         # Initialise the fsleyes.gl modules
         fslgl.bootstrap(namespace.glversion)
@@ -178,6 +181,13 @@ def makeDisplayContext(namespace):
     as described by the arguments in the given ``namespace`` object.
     """
 
+    # Set a display type hint. When running FSLeyes
+    # on-screen, this is retrieved automatically
+    # from the ViewPanel.displayType method, but we
+    # do it manually here for off-screen rendering.
+    if namespace.scene == '3d': displayType = '3D'
+    else:                       displayType = None
+
     # Create an overlay list and display context.
     # The DisplayContext, Display and DisplayOpts
     # classes are designed to be created in a
@@ -187,7 +197,8 @@ def makeDisplayContext(namespace):
     overlayList      = fsloverlay.OverlayList()
     masterDisplayCtx = displaycontext.DisplayContext(overlayList)
     childDisplayCtx  = displaycontext.DisplayContext(overlayList,
-                                                     parent=masterDisplayCtx)
+                                                     parent=masterDisplayCtx,
+                                                     displayType=displayType)
 
     # We have to artificially create a ref to the
     # master display context, otherwise it may get
@@ -376,27 +387,7 @@ def render(namespace, overlayList, displayCtx, sceneOpts, hook=None):
     for c in canvases:
 
         c.opts.pos = displayCtx.location
-
-        # HACK If a SliceCanvas/LightBoxCanvas
-        # is rendering the sceen to an off-screen
-        # texture due to the low performance
-        # setting, its internal viewport will not
-        # be set until after all GLObjects have
-        # been rendered. But some GLObjects (e.g.
-        # GLLabel) need to know the current
-        # viewport.
-        #
-        # This is very much an edge case, as who
-        # would be using a low performance setting
-        # for off-screen rendering?
-
-        if namespace.scene in ('ortho', 'lightbox') and \
-           namespace.performance is not None        and \
-           int(namespace.performance) < 3:
-            c._setViewport()
-
         c.draw()
-
         canvasBmps.append(c.getBitmap())
 
     # destroy the canvases
@@ -462,9 +453,9 @@ def createLightBoxCanvas(namespace,
 
     opts                = canvas.opts
     opts.showCursor     = sceneOpts.showCursor
+    opts.cursorWidth    = sceneOpts.cursorWidth
     opts.bgColour       = sceneOpts.bgColour
     opts.cursorColour   = sceneOpts.cursorColour
-    opts.renderMode     = sceneOpts.renderMode
     opts.zax            = sceneOpts.zax
     opts.sliceSpacing   = sceneOpts.sliceSpacing
     opts.nrows          = sceneOpts.nrows
@@ -562,9 +553,9 @@ def createOrthoCanvases(namespace,
         opts              = c.opts
         opts.showCursor   = sceneOpts.showCursor
         opts.cursorColour = sceneOpts.cursorColour
+        opts.cursorWidth  = sceneOpts.cursorWidth
         opts.cursorGap    = sceneOpts.cursorGap
         opts.bgColour     = sceneOpts.bgColour
-        opts.renderMode   = sceneOpts.renderMode
         opts.invertX      = invertx
         opts.invertY      = inverty
 
@@ -611,7 +602,6 @@ def create3DCanvas(namespace,
     opts.bgColour        = sceneOpts.bgColour
     opts.showLegend      = sceneOpts.showLegend
     opts.legendColour    = sceneOpts.fgColour
-    opts.occlusion       = sceneOpts.occlusion
     opts.light           = sceneOpts.light
     opts.zoom            = sceneOpts.zoom
     opts.offset          = sceneOpts.offset
