@@ -265,7 +265,7 @@ messages = TypeDict({
     'space to the image being edited!',
 
     'LoadVertexDataAction.loadVertexData' :
-    'Select a vertex data file for {}',
+    'Select a per-vertex/streamline data file for {}',
 
     'LoadVertexDataAction.loadVertices' :
     'Select a vertex file for {}',
@@ -723,12 +723,13 @@ labels = TypeDict({
     'OverlayDisplayPanel.TensorOpts'     : 'Diffusion tensor settings',
     'OverlayDisplayPanel.SHOpts'         : 'Diffusion SH settings',
     'OverlayDisplayPanel.MIPOpts'        : 'MIP settings',
+    'OverlayDisplayPanel.TractogramOpts' : 'Tractogram settings',
 
     'OverlayDisplayPanel.3d'             : '3D display settings',
 
-    'OverlayDisplayPanel.loadCmap'       : 'Load colour map',
-    'OverlayDisplayPanel.loadVertexData' : 'Load data',
-    'OverlayDisplayPanel.loadVertices'   : 'Load vertices',
+    'OverlayDisplayPanel.loadCmap'           : 'Load colour map',
+    'OverlayDisplayPanel.loadVertexData'     : 'Load vertex data',
+    'OverlayDisplayPanel.loadVertices'       : 'Load vertices',
 
     'OverlayDisplayPanel.clipPlane#'     : 'Clip plane #{}',
 
@@ -811,6 +812,12 @@ labels = TypeDict({
     'OverlayInfoPanel.MGHImage'                 : 'NIFTI Image (from MGH)',
     'OverlayInfoPanel.MGHImage.filename'        : 'MGH image file',
 
+    'OverlayInfoPanel.Tractogram'                : 'Trqctogram',
+    'OverlayInfoPanel.Tractogram.dataSource'     : 'Data source',
+    'OverlayInfoPanel.Tractogram.numStreamlines' : 'Total number of streamlines',
+    'OverlayInfoPanel.Tractogram.numVertices'    : 'Total number of vertices',
+    'OverlayInfoPanel.Tractogram.size'           : 'Bounding box size',
+
     'MelodicClassificationPanel.componentTab'   : 'Components',
     'MelodicClassificationPanel.labelTab'       : 'Labels',
     'MelodicClassificationPanel.loadButton'     : 'Load labels',
@@ -928,8 +935,9 @@ labels = TypeDict({
     'NewImageDialog.link'          : 'link with dimensions',
     'NewImageDialog.affine'        : 'Affine',
 
-    'XNATBrowser.ok'        : 'Download',
-    'XNATBrowser.cancel'    : 'Cancel',
+    'XNATBrowser.download'  : 'Download',
+    'XNATBrowser.close'     : 'Close',
+    'XNATBrowser.fullPaths' : 'Save full XNAT paths',
     'XNATBrowser.choosedir' : 'Choose a download location',
 
     'BrowseDicomPanel.dicomdir'          : 'Directory',
@@ -1011,12 +1019,12 @@ properties = TypeDict({
     'CanvasPanel.profile'            : 'Mode',
 
     'SceneOpts.showCursor'         : 'Show location cursor',
+    'SceneOpts.cursorWidth'        : 'Location cursor width',
     'SceneOpts.cursorGap'          : 'Show gap at cursor centre',
     'SceneOpts.bgColour'           : 'Background colour',
     'SceneOpts.fgColour'           : 'Foreground colour',
     'SceneOpts.cursorColour'       : 'Location cursor colour',
     'SceneOpts.showColourBar'      : 'Show colour bar',
-    'SceneOpts.performance'        : 'Rendering performance',
     'SceneOpts.zoom'               : 'Zoom',
     'SceneOpts.highDpi'            : 'Enable high-DPI rendering',
     'SceneOpts.colourBarLocation'  : 'Colour bar location',
@@ -1030,10 +1038,11 @@ properties = TypeDict({
     'LightBoxOpts.sliceSpacing'   : 'Slice spacing',
     'LightBoxOpts.zrange'         : 'Z range',
 
-    'OrthoOpts.showXCanvas' : 'Show X canvas',
-    'OrthoOpts.showYCanvas' : 'Show Y canvas',
-    'OrthoOpts.showZCanvas' : 'Show Z canvas',
-    'OrthoOpts.showLabels'  : 'Show labels',
+    'OrthoOpts.showXCanvas'  : 'Show X canvas',
+    'OrthoOpts.showYCanvas'  : 'Show Y canvas',
+    'OrthoOpts.showZCanvas'  : 'Show Z canvas',
+    'OrthoOpts.showLabels'   : 'Show labels',
+    'OrthoOpts.showLocation' : 'Show location',
 
     'OrthoOpts.layout'      : 'Layout',
     'OrthoOpts.xzoom'       : 'X zoom',
@@ -1045,7 +1054,6 @@ properties = TypeDict({
     'Scene3DOpts.showLight'     : 'Show light source',
     'Scene3DOpts.lightPos'      : 'Light position',
     'Scene3DOpts.lightDistance' : 'Light distance',
-    'Scene3DOpts.occlusion'     : 'Volume occlusion',
 
     'PlotCanvas.legend'     : 'Show legend',
     'PlotCanvas.ticks'      : 'Show ticks',
@@ -1144,22 +1152,24 @@ properties = TypeDict({
 
     'NiftiOpts.custom_volume' : 'Volume',
 
-    'ColourMapOpts.displayRange'     : 'Display range',
-    'ColourMapOpts.clippingRange'    : 'Clipping range',
-    'ColourMapOpts.modulateRange'    : 'Modulate range',
-    'ColourMapOpts.linkLowRanges'    : 'Link low display/clipping ranges',
-    'ColourMapOpts.linkHighRanges'   : 'Link high display/clipping ranges',
-    'ColourMapOpts.modulateAlpha'    : 'Modulate alpha by intensity',
-    'ColourMapOpts.cmap'             : 'Colour map',
-    'ColourMapOpts.custom_cmap'      : 'Colour map',
-    'ColourMapOpts.cmapResolution'   : 'Colour map resolution',
-    'ColourMapOpts.logScale'         : 'Logarithmic scaling',
-    'ColourMapOpts.gamma'            : 'Gamma correction',
-    'ColourMapOpts.interpolateCmaps' : 'Interpolate colour maps',
-    'ColourMapOpts.negativeCmap'     : '-ve colour map',
-    'ColourMapOpts.useNegativeCmap'  : '-ve colour map',
-    'ColourMapOpts.invert'           : 'Invert colour map',
-    'ColourMapOpts.invertClipping'   : 'Invert clipping range',
+    'ColourMapOpts.displayRange'         : 'Display range',
+    'ColourMapOpts.clippingRange'        : 'Clipping range',
+    'ColourMapOpts.modulateRange'        : 'Modulate range',
+    'ColourMapOpts.custom_linkRanges'    : 'Link display/clipping ranges',
+    'ColourMapOpts.linkLowRanges'        : 'Link low display/clipping ranges',
+    'ColourMapOpts.linkHighRanges'       : 'Link high display/clipping ranges',
+    'ColourMapOpts.modulateAlpha'        : 'Modulate alpha by intensity',
+    'ColourMapOpts.custom_modulateAlpha' : 'Modulate alpha by intensity',
+    'ColourMapOpts.cmap'                 : 'Colour map',
+    'ColourMapOpts.custom_cmap'          : 'Colour map',
+    'ColourMapOpts.cmapResolution'       : 'Colour map resolution',
+    'ColourMapOpts.logScale'             : 'Logarithmic scaling',
+    'ColourMapOpts.gamma'                : 'Gamma correction',
+    'ColourMapOpts.interpolateCmaps'     : 'Interpolate colour maps',
+    'ColourMapOpts.negativeCmap'         : '-ve colour map',
+    'ColourMapOpts.useNegativeCmap'      : '-ve colour map',
+    'ColourMapOpts.invert'               : 'Invert colour map',
+    'ColourMapOpts.invertClipping'       : 'Invert clipping range',
 
     'VolumeOpts.clipImage'                : 'Clip by',
     'VolumeOpts.modulateImage'            : 'Modulate by',
@@ -1269,12 +1279,20 @@ properties = TypeDict({
     'VolumeRGBOpts.suppressA'     : 'Suppress A',
     'VolumeRGBOpts.suppressMode'  : 'Suppress mode',
 
+    'TractogramOpts.clipMode'          : 'Clip by',
+    'TractogramOpts.custom_colourMode' : 'Colour by',
+    'TractogramOpts.colourMode'        : 'Colour by',
+    'TractogramOpts.lineWidth'         : 'Width',
+    'TractogramOpts.resolution'        : 'Resolution',
+    'TractogramOpts.subsample'         : 'Sub-sample',
 })
 
 
 choices = TypeDict({
 
-    'DisplayContext.displaySpace' : {'world'  : 'World coordinates'},
+    'DisplayContext.displaySpace' : {'world'       : 'World coordinates',
+                                     'scaledVoxel' : 'Scaled voxel coordinates',
+                                     },
 
     'SceneOpts.colourBarLocation'  : {'top'          : 'Top',
                                       'bottom'       : 'Bottom',
@@ -1283,10 +1301,6 @@ choices = TypeDict({
     'SceneOpts.colourBarLabelSide' : {'top-left'     : 'Top / Left',
                                       'bottom-right' : 'Bottom / Right'},
 
-    'SceneOpts.performance' : {1 : 'Fastest',
-                               2 : 'Faster',
-                               3 : 'Best looking'},
-
     'CanvasOpts.zax' : {0 : 'X axis',
                         1 : 'Y axis',
                         2 : 'Z axis'},
@@ -1295,6 +1309,12 @@ choices = TypeDict({
                           'vertical'   : 'Vertical',
                           'grid'       : 'Grid'},
 
+    'OrthoOpts.showLocation' : {'no' : 'No',
+                                'X'  : 'On X canvas',
+                                'Y'  : 'On Y canvas',
+                                'Z'  : 'On Z canvas'},
+
+
     'OrthoEditProfile.mode' : {'nav'    : 'Navigate',
                                'sel'    : 'Draw/select',
                                'desel'  : 'Erase/deselect',
@@ -1366,10 +1386,12 @@ choices = TypeDict({
     'SHOpts.colourMode' : {'radius'    : 'Colour by radius',
                            'direction' : 'Colour by direction'},
 
-    'ComplexOpts.component' :{ 'real'  : 'Real',
-                               'imag'  : 'Imaginary',
-                               'mag'   : 'Magnitude',
-                               'phase' : 'Phase'},
+    'ComplexOpts.component' : { 'real'  : 'Real',
+                                'imag'  : 'Imaginary',
+                                'mag'   : 'Magnitude',
+                                'phase' : 'Phase'},
+
+    'TractogramOpts.colourMode' : {'orientation' : 'Orientation'},
 
     'Display.overlayType' : {
         'volume'         : '3D/4D volume',
@@ -1383,6 +1405,7 @@ choices = TypeDict({
         'mesh'           : '3D mesh',
         'tensor'         : 'Diffusion tensor',
         'sh'             : 'Diffusion SH',
+        'tractogram'     : 'Streamline tractogram'
     },
 
     'HistogramPanel.histType' : {'probability' : 'Probability',
