@@ -388,10 +388,11 @@ properties = TypeDict({
     'filled.',
     'MeshOpts.lighting' :
     'When selected, a lighting effect is applied to the mesh.',
-    'MeshOpts.flatShading' :
-    'When selected, interpolation of the colours between adjacent vertices'
-    'is not performed - each triangle in the mesh is coloured according to '
-    'its first triangle.',
+    'MeshOpts.interpolation' :
+    'Interpolation of vertex data values - when linear interpolation is '
+    'selected, each triangle is coloured according to the vertex data values '
+    'at each vertex. When nearest neighbour interpolation is selected, each '
+    'triangle in the mesh is coloured according to its first triangle.',
 
     'TensorOpts.lighting'         : 'If enabled, a simple lighting model is '
                                     'used to highlight the tensor '
@@ -427,6 +428,24 @@ properties = TypeDict({
                                 'disabled when you choose to colour the FODs '
                                 'by another image (e.g. a FA map).',
 
+    'TractogramOpts.colourMode' :
+    'Tractograms can be coloured according to their orientation, according to '
+    'data taken fraom a NIfTI image, or according to per-vertex/streamline '
+    'values loaded from a file, or contained within the tractogram file.',
+    'TractogramOpts.clipMode' :
+    'Tractograms can be clipped according to data taken from a NIfTI image, '
+    'or according to  per-vertex/streamline values loaded from a file, or '
+    'contained within the tractogram file.',
+    'TractogramOpts.lineWidth' :
+    'Controls the thickness of streamline lines/tubes (when viewing in 3D), '
+    'or the diameter of streamline vertex points (when viewing in 2D).',
+    'TractogramOpts.resolution' :
+    'Controls the quality when drawing streamline tubes (in 3D) or vertex '
+    'points (in 2D).',
+    'TractogramOpts.subsample' :
+    'Draw a randomly selected subset of streamlines. This setting is useful '
+    'when viewing very large tractograms.',
+
     # MIPOpts
     'MIPOpts.window'  :
     'Length of the window, as a proportion of the image length, along which '
@@ -446,6 +465,7 @@ properties = TypeDict({
     # SceneOpts
     'SceneOpts.showCursor'         : 'Show/hide the cursor which highlights '
                                      'the current location.',
+    'SceneOpts.cursorWidth'        : 'Width/thickness of the location cursor.',
     'SceneOpts.cursorGap'          : 'Show a gap at the cursor centre.',
     'SceneOpts.cursorColour'       : 'Colour of the location cursor.',
     'SceneOpts.bgColour'           : 'Canvas background colour.',
@@ -464,12 +484,6 @@ properties = TypeDict({
     'SceneOpts.colourBarSize'      : 'Size of the major axis of the colour '
                                      'bar, as a proportion of the available '
                                      'space.',
-    'SceneOpts.performance'        : 'Rendering performance - 1 gives the '
-                                     'fastest, but at the cost of lower '
-                                     'display quality, and some display '
-                                     'limitations. 3 gives the best '
-                                     'display quality, but may be too slow on '
-                                     'some older systems.',
     'SceneOpts.highDpi' :
     'If you are using a high-DPI (e.g. retina) display, FSLeyes will attempt '
     'to display the scene at the high-DPI resolution. This will have an '
@@ -488,6 +502,8 @@ properties = TypeDict({
     'OrthoOpts.showLabels'  : 'If the currently selected overlay is a NIFTI '
                               'image, show / hide anatomical orientation '
                               'labels.',
+    'OrthoOpts.showLocation' :
+    'Show the cursor coordinates on one of the three canvases.',
     'OrthoOpts.layout'      : 'How to lay out each of the three canvases.',
     'OrthoOpts.zoom'        : 'Zoom level for all three canvases.',
 
@@ -525,9 +541,6 @@ properties = TypeDict({
     'Position of the light in the display coordinate system.',
     'Scene3DOpts.lightDistance' :
     'Distance of the light from the display centre.',
-    'Scene3DOpts.occlusion' :
-    'When selected, volumes in the scene which are behind another volume will '
-    'not be shown. ',
 
     'Scene3DOpts.zoom' :
     'Zoom level - distance from the camera to the model space.',
