@@ -258,6 +258,10 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         yopts.bindProps('cursorGap',    sceneOpts)
         zopts.bindProps('cursorGap',    sceneOpts)
 
+        xopts.bindProps('cursorWidth',  sceneOpts)
+        yopts.bindProps('cursorWidth',  sceneOpts)
+        zopts.bindProps('cursorWidth',  sceneOpts)
+
         xopts.bindProps('bgColour',     sceneOpts)
         yopts.bindProps('bgColour',     sceneOpts)
         zopts.bindProps('bgColour',     sceneOpts)
@@ -270,10 +274,6 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         yopts.bindProps('zoom',         sceneOpts, 'yzoom')
         zopts.bindProps('zoom',         sceneOpts, 'zzoom')
 
-        xopts.bindProps('renderMode',   sceneOpts)
-        yopts.bindProps('renderMode',   sceneOpts)
-        zopts.bindProps('renderMode',   sceneOpts)
-
         xopts.bindProps('highDpi',      sceneOpts)
         yopts.bindProps('highDpi',      sceneOpts)
         zopts.bindProps('highDpi',      sceneOpts)
@@ -320,9 +320,10 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         def refresh(*a):
             self.Refresh()
 
-        sceneOpts.addListener('labelSize',   name, refresh, weak=False)
-        sceneOpts.addListener('fgColour',    name, refresh, weak=False)
-        sceneOpts.addListener('showLabels',  name, refresh, weak=False)
+        sceneOpts.addListener('labelSize',    name, refresh, weak=False)
+        sceneOpts.addListener('fgColour',     name, refresh, weak=False)
+        sceneOpts.addListener('showLabels',   name, refresh, weak=False)
+        sceneOpts.addListener('showLocation', name, refresh, weak=False)
 
         # The lastFocusedCanvas method allows
         # a ref to the most recently focused
